# Copyright 2014 Cisco Systems, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# @author: Paul Michali, Cisco Systems, Inc.

#TODO(pcm): Rename this file to remove the "no" prefix, once httmock is
# approved and added to requirements.txt

import random

import httmock
import requests

from neutron.openstack.common import log as logging
from neutron.services.vpn.device_drivers import (
    cisco_csr_rest_client as csr_client)
from neutron.tests import base
from neutron.tests.unit.services.vpn.device_drivers import (
    cisco_csr_mock as csr_request)
# TODO(pcm) Remove once httmock is available. In the meantime, use temp
# copy of hhtmock source to run UT
# from neutron.tests.unit.services.vpn.device_drivers import httmock


LOG = logging.getLogger(__name__)
# Enables debug logging to console
if True:
    logging.CONF.set_override('debug', True)
    logging.setup('neutron')

if csr_request.FIXED_CSCum35484:
    dummy_uuid = '1eb4ee6b-0870-45a0-b554-7b69096'
else:
    dummy_uuid = '1eb4ee6b-0870-45a0-b554-7b'


# Note: Helper functions to test reuse of IDs.
def generate_pre_shared_key_id():
    return random.randint(100, 200)


def generate_ike_policy_id():
    return random.randint(200, 300)


def generate_ipsec_policy_id():
    return random.randint(300, 400)


class TestCsrLoginRestApi(base.BaseTestCase):

    """Test logging into CSR to obtain token-id."""

    def setUp(self):
        super(TestCsrLoginRestApi, self).setUp()
        self.csr = csr_client.CsrRestClient('localhost', '10.10.10.10',
                                            'stack', 'cisco')

    def test_get_token(self):
        """Obtain the token and its expiration time."""
        with httmock.HTTMock(csr_request.token):
            self.assertTrue(self.csr.authenticate())
            self.assertEqual(requests.codes.OK, self.csr.status)
            self.assertIsNotNone(self.csr.token)

    def test_unauthorized_token_request(self):
        """Negative test of invalid user/password."""
        self.csr.auth = ('stack', 'bogus')
        with httmock.HTTMock(csr_request.token_unauthorized):
            self.assertIsNone(self.csr.authenticate())
            self.assertEqual(requests.codes.UNAUTHORIZED, self.csr.status)

    def test_non_existent_host(self):
        """Negative test of request to non-existent host."""
        self.csr.host = 'wrong-host'
        self.csr.token = 'Set by some previously successful access'
        with httmock.HTTMock(csr_request.token_wrong_host):
            self.assertIsNone(self.csr.authenticate())
            self.assertEqual(requests.codes.NOT_FOUND, self.csr.status)
            self.assertIsNone(self.csr.token)

    def test_timeout_on_token_access(self):
        """Negative test of a timeout on a request."""
        with httmock.HTTMock(csr_request.token_timeout):
            self.assertIsNone(self.csr.authenticate())
            self.assertEqual(requests.codes.REQUEST_TIMEOUT, self.csr.status)
            self.assertIsNone(self.csr.token)


class TestCsrGetRestApi(base.BaseTestCase):

    """Test CSR GET REST API."""

    def setUp(self):
        super(TestCsrGetRestApi, self).setUp()
        self.csr = csr_client.CsrRestClient('localhost', '10.10.10.10',
                                            'stack', 'cisco')

    def test_valid_rest_gets(self):
        """Simple GET requests.

        First request will do a post to get token (login). Assumes
        that there are two interfaces on the CSR.
        """

        with httmock.HTTMock(csr_request.token, csr_request.normal_get):
            content = self.csr.get_request('global/host-name')
            self.assertEqual(requests.codes.OK, self.csr.status)
            self.assertIn('host-name', content)
            self.assertNotEqual(None, content['host-name'])

            content = self.csr.get_request('global/local-users')
            self.assertEqual(requests.codes.OK, self.csr.status)
            self.assertIn('users', content)


class TestCsrPostRestApi(base.BaseTestCase):

    """Test CSR POST REST API."""

    def setUp(self):
        super(TestCsrPostRestApi, self).setUp()
        self.csr = csr_client.CsrRestClient('localhost', '10.10.10.10',
                                            'stack', 'cisco')

    def test_post_requests(self):
        """Simple POST requests (repeatable).

        First request will do a post to get token (login). Assumes
        that there are two interfaces (Ge1 and Ge2) on the CSR.
        """

        with httmock.HTTMock(csr_request.token, csr_request.post):
            content = self.csr.post_request(
                'interfaces/GigabitEthernet1/statistics',
                payload={'action': 'clear'})
            self.assertEqual(requests.codes.NO_CONTENT, self.csr.status)
            self.assertIsNone(content)
            content = self.csr.post_request(
                'interfaces/GigabitEthernet2/statistics',
                payload={'action': 'clear'})
            self.assertEqual(requests.codes.NO_CONTENT, self.csr.status)
            self.assertIsNone(content)

    def test_post_with_location(self):
        """Create a user and verify that location returned."""
        with httmock.HTTMock(csr_request.token, csr_request.post):
            location = self.csr.post_request(
                'global/local-users',
                payload={'username': 'test-user',
                         'password': 'pass12345',
                         'privilege': 15})
            self.assertEqual(requests.codes.CREATED, self.csr.status)
            self.assertIn('global/local-users/test-user', location)

    def test_post_missing_required_attribute(self):
        """Negative test of POST with missing mandatory info."""
        with httmock.HTTMock(csr_request.token, csr_request.post):
            self.csr.post_request('global/local-users',
                                  payload={'password': 'pass12345',
                                           'privilege': 15})
            self.assertEqual(requests.codes.BAD_REQUEST, self.csr.status)

    def test_post_invalid_attribute(self):
        """Negative test of POST with invalid info."""
        with httmock.HTTMock(csr_request.token, csr_request.post):
            self.csr.post_request('global/local-users',
                                  payload={'username': 'test-user',
                                           'password': 'pass12345',
                                           'privilege': 20})
            self.assertEqual(requests.codes.BAD_REQUEST, self.csr.status)

    def test_post_already_exists(self):
        """Negative test of a duplicate POST.

        Uses the lower level _do_request() API to just perform the POST and
        obtain the response, without any error processing.
        """
        with httmock.HTTMock(csr_request.token, csr_request.post):
                location = self.csr._do_request(
                    'POST',
                    'global/local-users',
                    payload={'username': 'test-user',
                             'password': 'pass12345',
                             'privilege': 15},
                    more_headers=csr_client.HEADER_CONTENT_TYPE_JSON)
                self.assertEqual(requests.codes.CREATED, self.csr.status)
                self.assertIn('global/local-users/test-user', location)
        with httmock.HTTMock(csr_request.token,
                             csr_request.post_change_attempt):
                self.csr._do_request(
                    'POST',
                    'global/local-users',
                    payload={'username': 'test-user',
                             'password': 'pass12345',
                             'privilege': 15},
                    more_headers=csr_client.HEADER_CONTENT_TYPE_JSON)
                # Note: For local-user, a 404 error is returned. For
                # site-to-site connection a 400 is returned.
                self.assertEqual(requests.codes.NOT_FOUND, self.csr.status)

    def test_post_changing_value(self):
        """Negative test of a POST trying to change a value."""
        with httmock.HTTMock(csr_request.token, csr_request.post):
            location = self.csr.post_request(
                'global/local-users',
                payload={'username': 'test-user',
                         'password': 'pass12345',
                         'privilege': 15})
            self.assertEqual(requests.codes.CREATED, self.csr.status)
            self.assertIn('global/local-users/test-user', location)
        with httmock.HTTMock(csr_request.token,
                             csr_request.post_change_attempt):
            content = self.csr.post_request('global/local-users',
                                            payload={'username': 'test-user',
                                                     'password': 'changed',
                                                     'privilege': 15})
            self.assertEqual(requests.codes.NOT_FOUND, self.csr.status)
            expected = {u'error-code': -1,
                        u'error-message': u'user test-user already exists'}
            self.assertDictContainsSubset(expected, content)


class TestCsrPutRestApi(base.BaseTestCase):

    """Test CSR PUT REST API."""

    def _save_resources(self):
        with httmock.HTTMock(csr_request.token, csr_request.normal_get):
            details = self.csr.get_request('global/host-name')
            if self.csr.status != requests.codes.OK:
                self.fail("Unable to save original host name")
            self.original_host = details['host-name']
            details = self.csr.get_request('interfaces/GigabitEthernet1')
            if self.csr.status != requests.codes.OK:
                self.fail("Unable to save interface Ge1 description")
            self.original_if = details
            if details.get('description', ''):
                self.original_if['description'] = ''
            self.csr.token = None

    def _restore_resources(self, user, password):
        """Restore the host name and itnerface description.

        Must restore the user and password, so that authentication
        token can be obtained (as some tests corrupt auth info).
        Will also clear token, so that it gets a fresh token.
        """

        self.csr.auth = (user, password)
        self.csr.token = None
        with httmock.HTTMock(csr_request.token, csr_request.put):
            payload = {'host-name': self.original_host}
            self.csr.put_request('global/host-name', payload=payload)
            if self.csr.status != requests.codes.NO_CONTENT:
                self.fail("Unable to restore host name after test")
            payload = {'description': self.original_if['description'],
                       'if-name': self.original_if['if-name'],
                       'ip-address': self.original_if['ip-address'],
                       'subnet-mask': self.original_if['subnet-mask'],
                       'type': self.original_if['type']}
            self.csr.put_request('interfaces/GigabitEthernet1',
                                 payload=payload)
            if self.csr.status != requests.codes.NO_CONTENT:
                self.fail("Unable to restore I/F Ge1 description after test")

    def setUp(self):
        """Prepare for PUT API tests."""
        super(TestCsrPutRestApi, self).setUp()
        self.csr = csr_client.CsrRestClient('localhost', '10.10.10.10',
                                            'stack', 'cisco')
        self._save_resources()
        self.addCleanup(self._restore_resources, 'stack', 'cisco')

    def test_put_requests(self):
        """Simple PUT requests (repeatable).

        First request will do a post to get token (login). Assumes
        that there are two interfaces on the CSR (Ge1 and Ge2).
        """

        with httmock.HTTMock(csr_request.token, csr_request.put,
                             csr_request.normal_get):
            payload = {'host-name': 'TestHost'}
            content = self.csr.put_request('global/host-name',
                                           payload=payload)
            self.assertEqual(requests.codes.NO_CONTENT, self.csr.status)
            self.assertIsNone(content)

            payload = {'host-name': 'TestHost2'}
            content = self.csr.put_request('global/host-name',
                                           payload=payload)
            self.assertEqual(requests.codes.NO_CONTENT, self.csr.status)
            self.assertIsNone(content)

    def test_change_interface_description(self):
        """Test that interface description can be changed.

        This was a problem with an earlier version of the CSR image and is
        here to prevent regression.
        """
        with httmock.HTTMock(csr_request.token, csr_request.put,
                             csr_request.normal_get):
            payload = {'description': u'Changed description',
                       'if-name': self.original_if['if-name'],
                       'ip-address': self.original_if['ip-address'],
                       'subnet-mask': self.original_if['subnet-mask'],
                       'type': self.original_if['type']}
            content = self.csr.put_request(
                'interfaces/GigabitEthernet1', payload=payload)
            self.assertEqual(requests.codes.NO_CONTENT, self.csr.status)
            self.assertIsNone(content)
            content = self.csr.get_request('interfaces/GigabitEthernet1')
            self.assertEqual(requests.codes.OK, self.csr.status)
            self.assertIn('description', content)
            self.assertEqual(u'Changed description',
                             content['description'])

    def ignore_test_change_to_empty_interface_description(self):
        """Test that interface description can be changed to empty string.

        This is a problem in the current version of the CSR image, which
        rejects the change with a 400 error. This test is here to prevent
        a regression (once it is fixed) Note that there is code in the
        test setup to change the description to a non-empty string to
        avoid failures in other tests.
        """
        with httmock.HTTMock(csr_request.token, csr_request.put,
                             csr_request.normal_get):
            payload = {'description': '',
                       'if-name': self.original_if['if-name'],
                       'ip-address': self.original_if['ip-address'],
                       'subnet-mask': self.original_if['subnet-mask'],
                       'type': self.original_if['type']}
            content = self.csr.put_request(
                'interfaces/GigabitEthernet1', payload=payload)
            self.assertEqual(requests.codes.NO_CONTENT, self.csr.status)
            self.assertIsNone(content)
            content = self.csr.get_request('interfaces/GigabitEthernet1')
            self.assertEqual(requests.codes.OK, self.csr.status)
            self.assertIn('description', content)
            self.assertEqual('', content['description'])


class TestCsrDeleteRestApi(base.BaseTestCase):

    """Test CSR DELETE REST API."""

    def setUp(self):
        super(TestCsrDeleteRestApi, self).setUp()
        self.csr = csr_client.CsrRestClient('localhost', '10.10.10.10',
                                            'stack', 'cisco')

    def _make_dummy_user(self):
        """Create a user that will be later deleted."""
        self.csr.post_request('global/local-users',
                              payload={'username': 'dummy',
                                       'password': 'dummy',
                                       'privilege': 15})
        self.assertEqual(requests.codes.CREATED, self.csr.status)

    def test_delete_requests(self):
        """Simple DELETE requests (creating entry first)."""
        with httmock.HTTMock(csr_request.token, csr_request.post,
                             csr_request.delete):
            self._make_dummy_user()
            self.csr.token = None  # Force login
            self.csr.delete_request('global/local-users/dummy')
            self.assertEqual(requests.codes.NO_CONTENT, self.csr.status)
            # Delete again, but without logging in this time
            self._make_dummy_user()
            self.csr.delete_request('global/local-users/dummy')
            self.assertEqual(requests.codes.NO_CONTENT, self.csr.status)

    def test_delete_non_existent_entry(self):
        """Negative test of trying to delete a non-existent user."""
        with httmock.HTTMock(csr_request.token, csr_request.delete_unknown):
            content = self.csr.delete_request('global/local-users/unknown')
            self.assertEqual(requests.codes.NOT_FOUND, self.csr.status)
            expected = {u'error-code': -1,
                        u'error-message': u'user unknown not found'}
            self.assertDictContainsSubset(expected, content)

    def test_delete_not_allowed(self):
        """Negative test of trying to delete the host-name."""
        with httmock.HTTMock(csr_request.token,
                             csr_request.delete_not_allowed):
            self.csr.delete_request('global/host-name')
            self.assertEqual(requests.codes.METHOD_NOT_ALLOWED,
                             self.csr.status)


class TestCsrRestApiFailures(base.BaseTestCase):

    """Test failure cases common for all REST APIs.

    Uses the lower level _do_request() to just perform the operation and get
    the result, without any error handling.
    """

    def setUp(self):
        super(TestCsrRestApiFailures, self).setUp()
        self.csr = csr_client.CsrRestClient('localhost', '10.10.10.10',
                                            'stack', 'cisco', timeout=0.1)

    def test_request_for_non_existent_resource(self):
        """Negative test of non-existent resource on REST request."""
        with httmock.HTTMock(csr_request.token, csr_request.no_such_resource):
            self.csr.post_request('no/such/request')
            self.assertEqual(requests.codes.NOT_FOUND, self.csr.status)
            # The result is HTTP 404 message, so no error content to check

    def test_timeout_during_request(self):
        """Negative test of timeout during REST request."""
        with httmock.HTTMock(csr_request.token, csr_request.timeout):
            self.csr._do_request('GET', 'global/host-name')
            self.assertEqual(requests.codes.REQUEST_TIMEOUT, self.csr.status)

    def test_token_expired_on_request(self):
        """Token expired before trying a REST request.

        The mock is configured to return a 401 error on the first
        attempt to reference the host name. Simulate expiration of
        token by changing it.
        """

        with httmock.HTTMock(csr_request.token, csr_request.expired_request,
                             csr_request.normal_get):
            self.csr.token = '123'  # These are 44 characters, so won't match
            content = self.csr._do_request('GET', 'global/host-name')
            self.assertEqual(requests.codes.OK, self.csr.status)
            self.assertIn('host-name', content)
            self.assertNotEqual(None, content['host-name'])

    def test_failed_to_obtain_token_for_request(self):
        """Negative test of unauthorized user for REST request."""
        self.csr.auth = ('stack', 'bogus')
        with httmock.HTTMock(csr_request.token_unauthorized):
            self.csr._do_request('GET', 'global/host-name')
            self.assertEqual(requests.codes.UNAUTHORIZED, self.csr.status)


class TestCsrRestIkePolicyCreate(base.BaseTestCase):

    """Test IKE policy create REST requests."""

    def setUp(self):
        super(TestCsrRestIkePolicyCreate, self).setUp()
        self.csr = csr_client.CsrRestClient('localhost', '10.10.10.10',
                                            'stack', 'cisco')

    def test_create_delete_ike_policy(self):
        """Create and then delete IKE policy."""
        with httmock.HTTMock(csr_request.token, csr_request.post,
                             csr_request.normal_get):
            policy_id = '2'
            policy_info = {u'priority-id': u'%s' % policy_id,
                           u'encryption': u'aes',
                           u'hash': u'sha',
                           u'dhGroup': 5,
                           u'lifetime': 3600}
            location = self.csr.create_ike_policy(policy_info)
            self.assertEqual(requests.codes.CREATED, self.csr.status)
            self.assertIn('vpn-svc/ike/policies/%s' % policy_id, location)
            # Check the hard-coded items that get set as well...
            content = self.csr.get_request(location, full_url=True)
            self.assertEqual(requests.codes.OK, self.csr.status)
            expected_policy = {u'kind': u'object#ike-policy',
                               u'version': u'v1',
                               u'local-auth-method': u'pre-share'}
            expected_policy.update(policy_info)
            self.assertEqual(expected_policy, content)
        # Now delete and verify the IKE policy is gone
        with httmock.HTTMock(csr_request.token, csr_request.delete,
                             csr_request.no_such_resource):
            self.csr.delete_ike_policy(policy_id)
            self.assertEqual(requests.codes.NO_CONTENT, self.csr.status)
            content = self.csr.get_request(location, full_url=True)
            self.assertEqual(requests.codes.NOT_FOUND, self.csr.status)

    def test_create_ike_policy_with_defaults(self):
        """Create IKE policy using defaults for all optional values."""
        with httmock.HTTMock(csr_request.token, csr_request.post,
                             csr_request.get_defaults):
            policy_id = '2'
            policy_info = {u'priority-id': u'%s' % policy_id}
            location = self.csr.create_ike_policy(policy_info)
            self.assertEqual(requests.codes.CREATED, self.csr.status)
            self.assertIn('vpn-svc/ike/policies/%s' % policy_id, location)
            # Check the hard-coded items that get set as well...
            content = self.csr.get_request(location, full_url=True)
            self.assertEqual(requests.codes.OK, self.csr.status)
            expected_policy = {u'kind': u'object#ike-policy',
                               u'version': u'v1',
                               u'encryption': u'des',
                               u'hash': u'sha',
                               u'dhGroup': 1,
                               u'lifetime': 86400,
                               # Lower level sets this, but it is the default
                               u'local-auth-method': u'pre-share'}
            expected_policy.update(policy_info)
            self.assertEqual(expected_policy, content)

    def test_create_duplicate_ike_policy(self):
        """Negative test of trying to create a duplicate IKE policy."""
        with httmock.HTTMock(csr_request.token, csr_request.post,
                             csr_request.normal_get):
            policy_id = '2'
            policy_info = {u'priority-id': u'%s' % policy_id,
                           u'encryption': u'aes',
                           u'hash': u'sha',
                           u'dhGroup': 5,
                           u'lifetime': 3600}
            location = self.csr.create_ike_policy(policy_info)
            self.assertEqual(requests.codes.CREATED, self.csr.status)
            self.assertIn('vpn-svc/ike/policies/%s' % policy_id, location)
        with httmock.HTTMock(csr_request.token, csr_request.post_duplicate):
            location = self.csr.create_ike_policy(policy_info)
            self.assertEqual(requests.codes.BAD_REQUEST, self.csr.status)
            expected = {u'error-code': -1,
                        u'error-message': u'policy 2 exist, not allow to '
                        u'update policy using POST method'}
            self.assertDictContainsSubset(expected, location)


class TestCsrRestIPSecPolicyCreate(base.BaseTestCase):

    """Test IPSec policy create REST requests."""

    def setUp(self):
        super(TestCsrRestIPSecPolicyCreate, self).setUp()
        self.csr = csr_client.CsrRestClient('localhost', '10.10.10.10',
                                            'stack', 'cisco')

    def test_create_delete_ipsec_policy(self):
        """Create and then delete IPSec policy."""
        with httmock.HTTMock(csr_request.token, csr_request.post,
                             csr_request.normal_get):
            policy_id = '123'
            policy_info = {
                u'policy-id': u'%s' % policy_id,
                u'protection-suite': {
                    u'esp-encryption': u'esp-aes',
                    u'esp-authentication': u'esp-sha-hmac',
                    u'ah': u'ah-sha-hmac',
                },
                u'lifetime-sec': 120,
                u'pfs': u'group5',
                u'anti-replay-window-size': u'128'
            }
            location = self.csr.create_ipsec_policy(policy_info)
            self.assertEqual(requests.codes.CREATED, self.csr.status)
            self.assertIn('vpn-svc/ipsec/policies/%s' % policy_id, location)
            # Check the hard-coded items that get set as well...
            content = self.csr.get_request(location, full_url=True)
            self.assertEqual(requests.codes.OK, self.csr.status)
            expected_policy = {u'kind': u'object#ipsec-policy',
                               u'mode': u'tunnel',
                               u'lifetime-kb': 4608000,
                               u'idle-time': None}
            expected_policy.update(policy_info)
            self.assertEqual(expected_policy, content)
        # Now delete and verify the IPSec policy is gone
        with httmock.HTTMock(csr_request.token, csr_request.delete,
                             csr_request.no_such_resource):
            self.csr.delete_ipsec_policy(policy_id)
            self.assertEqual(requests.codes.NO_CONTENT, self.csr.status)
            content = self.csr.get_request(location, full_url=True)
            self.assertEqual(requests.codes.NOT_FOUND, self.csr.status)

    def test_create_ipsec_policy_with_defaults(self):
        """Create IPSec policy with default for all optional values."""
        with httmock.HTTMock(csr_request.token, csr_request.post,
                             csr_request.get_defaults):
            policy_id = '123'
            policy_info = {
                u'policy-id': u'%s' % policy_id,
            }
            location = self.csr.create_ipsec_policy(policy_info)
            self.assertEqual(requests.codes.CREATED, self.csr.status)
            self.assertIn('vpn-svc/ipsec/policies/%s' % policy_id, location)
            # Check the hard-coded items that get set as well...
            content = self.csr.get_request(location, full_url=True)
            self.assertEqual(requests.codes.OK, self.csr.status)
            expected_policy = {u'kind': u'object#ipsec-policy',
                               u'mode': u'tunnel',
                               u'protection-suite': {},
                               u'lifetime-sec': 3600,
                               u'pfs': u'Disable',
                               u'anti-replay-window-size': u'None',
                               u'lifetime-kb': 4608000,
                               u'idle-time': None}
            expected_policy.update(policy_info)
            self.assertEqual(expected_policy, content)

    def test_create_ipsec_policy_with_uuid(self):
        """Create IPSec policy using UUID for id."""
        with httmock.HTTMock(csr_request.token, csr_request.post,
                             csr_request.normal_get):
            policy_info = {
                u'policy-id': u'%s' % dummy_uuid,
                u'protection-suite': {
                    u'esp-encryption': u'esp-aes',
                    u'esp-authentication': u'esp-sha-hmac',
                    u'ah': u'ah-sha-hmac',
                },
                u'lifetime-sec': 120,
                u'pfs': u'group5',
                u'anti-replay-window-size': u'128'
            }
            location = self.csr.create_ipsec_policy(policy_info)
            self.assertEqual(requests.codes.CREATED, self.csr.status)
            self.assertIn('vpn-svc/ipsec/policies/%s' % dummy_uuid,
                          location)
            # Check the hard-coded items that get set as well...
            content = self.csr.get_request(location, full_url=True)
            self.assertEqual(requests.codes.OK, self.csr.status)
            expected_policy = {u'kind': u'object#ipsec-policy',
                               u'mode': u'tunnel',
                               u'lifetime-kb': 4608000,
                               u'idle-time': None}
            expected_policy.update(policy_info)
            self.assertEqual(expected_policy, content)

    def test_create_ipsec_policy_without_ah(self):
        """Create IPSec policy."""
        with httmock.HTTMock(csr_request.token, csr_request.post,
                             csr_request.get_no_ah):
            policy_id = '10'
            policy_info = {
                u'policy-id': u'%s' % policy_id,
                u'protection-suite': {
                    u'esp-encryption': u'esp-aes',
                    u'esp-authentication': u'esp-sha-hmac',
                },
                u'lifetime-sec': 120,
                u'pfs': u'group5',
                u'anti-replay-window-size': u'128'
            }
            location = self.csr.create_ipsec_policy(policy_info)
            self.assertEqual(requests.codes.CREATED, self.csr.status)
            self.assertIn('vpn-svc/ipsec/policies/%s' % policy_id, location)
            # Check the hard-coded items that get set as well...
            content = self.csr.get_request(location, full_url=True)
            self.assertEqual(requests.codes.OK, self.csr.status)
            expected_policy = {u'kind': u'object#ipsec-policy',
                               u'mode': u'tunnel',
                               u'lifetime-kb': 4608000,
                               u'idle-time': None}
            expected_policy.update(policy_info)
            self.assertEqual(expected_policy, content)

    def test_invalid_ipsec_policy_lifetime(self):
        """Failure test of IPSec policy with unsupported lifetime."""
        with httmock.HTTMock(csr_request.token,
                             csr_request.post_bad_lifetime):
            policy_id = '123'
            policy_info = {
                u'policy-id': u'%s' % policy_id,
                u'protection-suite': {
                    u'esp-encryption': u'esp-aes',
                    u'esp-authentication': u'esp-sha-hmac',
                    u'ah': u'ah-sha-hmac',
                },
                u'lifetime-sec': 119,
                u'pfs': u'group5',
                u'anti-replay-window-size': u'128'
            }
            self.csr.create_ipsec_policy(policy_info)
            self.assertEqual(requests.codes.BAD_REQUEST, self.csr.status)


class TestCsrRestPreSharedKeyCreate(base.BaseTestCase):

    """Test Pre-shared key (PSK) create REST requests."""

    def setUp(self):
        super(TestCsrRestPreSharedKeyCreate, self).setUp()
        self.csr = csr_client.CsrRestClient('localhost', '10.10.10.10',
                                            'stack', 'cisco')

    def test_create_delete_pre_shared_key(self):
        """Create and then delete a keyring entry for pre-shared key."""
        with httmock.HTTMock(csr_request.token, csr_request.post,
                             csr_request.normal_get):
            psk_id = '5'
            psk_info = {u'keyring-name': u'%s' % psk_id,
                        u'pre-shared-key-list': [
                            {u'key': u'super-secret',
                             u'encrypted': False,
                             u'peer-address': u'10.10.10.20/24'}
                        ]}
            location = self.csr.create_pre_shared_key(psk_info)
            self.assertEqual(requests.codes.CREATED, self.csr.status)
            self.assertIn('vpn-svc/ike/keyrings/%s' % psk_id, location)
            # Check the hard-coded items that get set as well...
            content = self.csr.get_request(location, full_url=True)
            self.assertEqual(requests.codes.OK, self.csr.status)
            expected_policy = {u'kind': u'object#ike-keyring'}
            expected_policy.update(psk_info)
            # Note: the peer CIDR is returned as an IP and mask
            expected_policy[u'pre-shared-key-list'][0][u'peer-address'] = (
                u'10.10.10.20 255.255.255.0')
            self.assertEqual(expected_policy, content)
        # Now delete and verify pre-shared key is gone
        with httmock.HTTMock(csr_request.token, csr_request.delete,
                             csr_request.no_such_resource):
            self.csr.delete_pre_shared_key(psk_id)
            self.assertEqual(requests.codes.NO_CONTENT, self.csr.status)
            content = self.csr.get_request(location, full_url=True)
            self.assertEqual(requests.codes.NOT_FOUND, self.csr.status)

    def test_create_pre_shared_key_with_fqdn_peer(self):
        """Create pre-shared key using FQDN for peer address."""
        with httmock.HTTMock(csr_request.token, csr_request.post,
                             csr_request.get_fqdn):
            psk_id = '5'
            psk_info = {u'keyring-name': u'%s' % psk_id,
                        u'pre-shared-key-list': [
                            {u'key': u'super-secret',
                             u'encrypted': False,
                             u'peer-address': u'cisco.com'}
                        ]}
            location = self.csr.create_pre_shared_key(psk_info)
            self.assertEqual(requests.codes.CREATED, self.csr.status)
            self.assertIn('vpn-svc/ike/keyrings/%s' % psk_id, location)
            # Check the hard-coded items that get set as well...
            content = self.csr.get_request(location, full_url=True)
            self.assertEqual(requests.codes.OK, self.csr.status)
            expected_policy = {u'kind': u'object#ike-keyring'}
            expected_policy.update(psk_info)
            self.assertEqual(expected_policy, content)

    def test_create_pre_shared_key_with_duplicate_peer_address(self):
        """Negative test of creating a second pre-shared key with same peer."""
        with httmock.HTTMock(csr_request.token, csr_request.post,
                             csr_request.normal_get):
            psk_id = '5'
            psk_info = {u'keyring-name': u'%s' % psk_id,
                        u'pre-shared-key-list': [
                            {u'key': u'super-secret',
                             u'encrypted': False,
                             u'peer-address': u'10.10.10.20/24'}
                        ]}
            location = self.csr.create_pre_shared_key(psk_info)
            self.assertEqual(requests.codes.CREATED, self.csr.status)
            self.assertIn('vpn-svc/ike/keyrings/%s' % psk_id, location)
        with httmock.HTTMock(csr_request.token, csr_request.post_duplicate):
            psk_id = u'6'
            another_psk_info = {u'keyring-name': psk_id,
                                u'pre-shared-key-list': [
                                    {u'key': u'abc123def',
                                     u'encrypted': False,
                                     u'peer-address': u'10.10.10.20/24'}
                                ]}
            self.csr.create_ike_policy(another_psk_info)
            self.assertEqual(requests.codes.BAD_REQUEST, self.csr.status)


class TestCsrRestIPSecConnectionCreate(base.BaseTestCase):

    """Test IPSec site-to-site connection REST requests.

    This requires us to have first created an IKE policy, IPSec policy,
    and pre-shared key, so it's more of an itegration test, when used
    with a real CSR (as we can't mock out these pre-conditions.
    """

    def setUp(self):
        super(TestCsrRestIPSecConnectionCreate, self).setUp()
        self.csr = csr_client.CsrRestClient('localhost', '10.10.10.10',
                                            'stack', 'cisco')

    def _make_psk_for_test(self):
        psk_id = generate_pre_shared_key_id()
        self._remove_resource_for_test(self.csr.delete_pre_shared_key,
                                       psk_id)
        with httmock.HTTMock(csr_request.token, csr_request.post):
            psk_info = {u'keyring-name': u'%d' % psk_id,
                        u'pre-shared-key-list': [
                            {u'key': u'super-secret',
                             u'encrypted': False,
                             u'peer-address': u'10.10.10.20/24'}
                        ]}
            self.csr.create_pre_shared_key(psk_info)
            if self.csr.status != requests.codes.CREATED:
                self.fail("Unable to create PSK for test case")
            self.addCleanup(self._remove_resource_for_test,
                            self.csr.delete_pre_shared_key, psk_id)
            return psk_id

    def _make_ike_policy_for_test(self):
        policy_id = generate_ike_policy_id()
        self._remove_resource_for_test(self.csr.delete_ike_policy,
                                       policy_id)
        with httmock.HTTMock(csr_request.token, csr_request.post):
            policy_info = {u'priority-id': u'%d' % policy_id,
                           u'encryption': u'aes',
                           u'hash': u'sha',
                           u'dhGroup': 5,
                           u'lifetime': 3600}
            self.csr.create_ike_policy(policy_info)
            if self.csr.status != requests.codes.CREATED:
                self.fail("Unable to create IKE policy for test case")
            self.addCleanup(self._remove_resource_for_test,
                            self.csr.delete_ike_policy, policy_id)
            return policy_id

    def _make_ipsec_policy_for_test(self):
        policy_id = generate_ipsec_policy_id()
        self._remove_resource_for_test(self.csr.delete_ipsec_policy,
                                       policy_id)
        with httmock.HTTMock(csr_request.token, csr_request.post):
            policy_info = {
                u'policy-id': u'%d' % policy_id,
                u'protection-suite': {
                    u'esp-encryption': u'esp-aes',
                    u'esp-authentication': u'esp-sha-hmac',
                    u'ah': u'ah-sha-hmac',
                },
                u'lifetime-sec': 120,
                u'pfs': u'group5',
                u'anti-replay-window-size': u'64'
            }
            self.csr.create_ipsec_policy(policy_info)
            if self.csr.status != requests.codes.CREATED:
                self.fail("Unable to create IPSec policy for test case")
            self.addCleanup(self._remove_resource_for_test,
                            self.csr.delete_ipsec_policy, policy_id)
            return policy_id

    def _remove_resource_for_test(self, delete_resource, resource_id):
        with httmock.HTTMock(csr_request.token, csr_request.delete):
            delete_resource(resource_id)

    def _prepare_for_site_conn_create(self, skip_psk=False, skip_ike=False,
                                      skip_ipsec=False):
        """Create the policies and PSK so can then create site conn."""
        if not skip_psk:
            self._make_psk_for_test()
        if not skip_ike:
            self._make_ike_policy_for_test()
        if not skip_ipsec:
            ipsec_policy_id = self._make_ipsec_policy_for_test()
        else:
            ipsec_policy_id = generate_ipsec_policy_id()
        # Note: Use same ID number for tunnel and IPSec policy, so that when
        # GET tunnel info, the mocks can infer the IPSec policy ID from the
        # tunnel number.
        return (ipsec_policy_id, ipsec_policy_id)

    def test_create_delete_ipsec_connection(self):
        """Create and then delete an IPSec connection."""
        tunnel_id, ipsec_policy_id = self._prepare_for_site_conn_create()
        with httmock.HTTMock(csr_request.token, csr_request.post,
                             csr_request.normal_get):
            connection_info = {
                u'vpn-interface-name': u'Tunnel%d' % tunnel_id,
                u'ipsec-policy-id': u'%d' % ipsec_policy_id,
                u'mtu': 1500,
                u'local-device': {u'ip-address': u'10.3.0.1/24',
                                  u'tunnel-ip-address': u'10.10.10.10'},
                u'remote-device': {u'tunnel-ip-address': u'10.10.10.20'}
            }
            location = self.csr.create_ipsec_connection(connection_info)
            self.addCleanup(self._remove_resource_for_test,
                            self.csr.delete_ipsec_connection,
                            'Tunnel%d' % tunnel_id)
            self.assertEqual(requests.codes.CREATED, self.csr.status)
            self.assertIn('vpn-svc/site-to-site/Tunnel%d' % tunnel_id,
                          location)
            # Check the hard-coded items that get set as well...
            content = self.csr.get_request(location, full_url=True)
            self.assertEqual(requests.codes.OK, self.csr.status)
            expected_connection = {u'kind': u'object#vpn-site-to-site',
                                   u'ike-profile-id': None,
                                   u'mtu': 1500,
                                   u'ip-version': u'ipv4'}
            expected_connection.update(connection_info)
            self.assertEqual(expected_connection, content)
        # Now delete and verify that site-to-site connection is gone
        with httmock.HTTMock(csr_request.token, csr_request.delete,
                             csr_request.no_such_resource):
            # Only delete connection. Cleanup will take care of prerequisites
            self.csr.delete_ipsec_connection('Tunnel%d' % tunnel_id)
            self.assertEqual(requests.codes.NO_CONTENT, self.csr.status)
            content = self.csr.get_request(location, full_url=True)
            self.assertEqual(requests.codes.NOT_FOUND, self.csr.status)

    def test_create_ipsec_connection_with_no_tunnel_subnet(self):
        """Create an IPSec connection without an IP address on tunnel."""
        tunnel_id, ipsec_policy_id = self._prepare_for_site_conn_create()
        with httmock.HTTMock(csr_request.token, csr_request.post,
                             csr_request.get_unnumbered):
            connection_info = {
                u'vpn-interface-name': u'Tunnel%d' % tunnel_id,
                u'ipsec-policy-id': u'%d' % ipsec_policy_id,
                u'local-device': {u'ip-address': u'GigabitEthernet3',
                                  u'tunnel-ip-address': u'10.10.10.10'},
                u'remote-device': {u'tunnel-ip-address': u'10.10.10.20'}
            }
            location = self.csr.create_ipsec_connection(connection_info)
            self.addCleanup(self._remove_resource_for_test,
                            self.csr.delete_ipsec_connection,
                            'Tunnel%d' % tunnel_id)
            self.assertEqual(requests.codes.CREATED, self.csr.status)
            self.assertIn('vpn-svc/site-to-site/Tunnel%d' % tunnel_id,
                          location)
            # Check the hard-coded items that get set as well...
            content = self.csr.get_request(location, full_url=True)
            self.assertEqual(requests.codes.OK, self.csr.status)
            expected_connection = {u'kind': u'object#vpn-site-to-site',
                                   u'ike-profile-id': None,
                                   u'mtu': 1500,
                                   u'ip-version': u'ipv4'}
            expected_connection.update(connection_info)
            self.assertEqual(expected_connection, content)

    def test_create_ipsec_connection_no_pre_shared_key(self):
        """Test of connection create without associated pre-shared key.

        The CSR will create the connection, but will not be able to pass
        traffic without the pre-shared key.
        """

        tunnel_id, ipsec_policy_id = self._prepare_for_site_conn_create(
            skip_psk=True)
        with httmock.HTTMock(csr_request.token, csr_request.post,
                             csr_request.normal_get):
            connection_info = {
                u'vpn-interface-name': u'Tunnel%d' % tunnel_id,
                u'ipsec-policy-id': u'%d' % ipsec_policy_id,
                u'mtu': 1500,
                u'local-device': {u'ip-address': u'10.3.0.1/24',
                                  u'tunnel-ip-address': u'10.10.10.10'},
                u'remote-device': {u'tunnel-ip-address': u'10.10.10.20'}
            }
            location = self.csr.create_ipsec_connection(connection_info)
            self.addCleanup(self._remove_resource_for_test,
                            self.csr.delete_ipsec_connection,
                            'Tunnel%d' % tunnel_id)
            self.assertEqual(requests.codes.CREATED, self.csr.status)
            self.assertIn('vpn-svc/site-to-site/Tunnel%d' % tunnel_id,
                          location)
            # Check the hard-coded items that get set as well...
            content = self.csr.get_request(location, full_url=True)
            self.assertEqual(requests.codes.OK, self.csr.status)
            expected_connection = {u'kind': u'object#vpn-site-to-site',
                                   u'ike-profile-id': None,
                                   u'mtu': 1500,
                                   u'ip-version': u'ipv4'}
            expected_connection.update(connection_info)
            self.assertEqual(expected_connection, content)

    def test_create_ipsec_connection_with_default_ike_policy(self):
        """Test of connection create without IKE policy (uses default).

        Without an IKE policy, the CSR will use a built-in default IKE
        policy setting for the connection.
        """

        tunnel_id, ipsec_policy_id = self._prepare_for_site_conn_create(
            skip_ike=True)
        with httmock.HTTMock(csr_request.token, csr_request.post,
                             csr_request.normal_get):
            connection_info = {
                u'vpn-interface-name': u'Tunnel%d' % tunnel_id,
                u'ipsec-policy-id': u'%d' % ipsec_policy_id,
                u'mtu': 1500,
                u'local-device': {u'ip-address': u'10.3.0.1/24',
                                  u'tunnel-ip-address': u'10.10.10.10'},
                u'remote-device': {u'tunnel-ip-address': u'10.10.10.20'}
            }
            location = self.csr.create_ipsec_connection(connection_info)
            self.addCleanup(self._remove_resource_for_test,
                            self.csr.delete_ipsec_connection,
                            'Tunnel%d' % tunnel_id)
            self.assertEqual(requests.codes.CREATED, self.csr.status)
            self.assertIn('vpn-svc/site-to-site/Tunnel%d' % tunnel_id,
                          location)
            # Check the hard-coded items that get set as well...
            content = self.csr.get_request(location, full_url=True)
            self.assertEqual(requests.codes.OK, self.csr.status)
            expected_connection = {u'kind': u'object#vpn-site-to-site',
                                   u'ike-profile-id': None,
                                   u'mtu': 1500,
                                   u'ip-version': u'ipv4'}
            expected_connection.update(connection_info)
            self.assertEqual(expected_connection, content)

    def test_create_ipsec_connection_missing_ipsec_policy(self):
        """Negative test of connection create without IPSec policy."""
        tunnel_id, ipsec_policy_id = self._prepare_for_site_conn_create(
            skip_ipsec=True)
        with httmock.HTTMock(csr_request.token,
                             csr_request.post_missing_ipsec_policy):
            connection_info = {
                u'vpn-interface-name': u'Tunnel%d' % tunnel_id,
                u'ipsec-policy-id': u'%d' % ipsec_policy_id,
                u'local-device': {u'ip-address': u'10.3.0.1/24',
                                  u'tunnel-ip-address': u'10.10.10.10'},
                u'remote-device': {u'tunnel-ip-address': u'10.10.10.20'}
            }
            self.csr.create_ipsec_connection(connection_info)
            self.addCleanup(self._remove_resource_for_test,
                            self.csr.delete_ipsec_connection,
                            'Tunnel%d' % tunnel_id)
            self.assertEqual(requests.codes.BAD_REQUEST, self.csr.status)

    def _determine_conflicting_ip(self):
        with httmock.HTTMock(csr_request.token, csr_request.get_local_ip):
            details = self.csr.get_request('interfaces/GigabitEthernet3')
            if self.csr.status != requests.codes.OK:
                self.fail("Unable to obtain interface GigabitEthernet3's IP")
            if_ip = details.get('ip-address')
            if not if_ip:
                self.fail("No IP address for GigabitEthernet3 interface")
            return '.'.join(if_ip.split('.')[:3]) + '.10'

    def test_create_ipsec_connection_conficting_tunnel_ip(self):
        """Negative test of connection create with conflicting tunnel IP.

        Find out the IP of a local interface (GigabitEthernet3) and create an
        IP that is on the same subnet. Note: this interface needs to be up.
        """

        conflicting_ip = self._determine_conflicting_ip()
        tunnel_id, ipsec_policy_id = self._prepare_for_site_conn_create()
        with httmock.HTTMock(csr_request.token, csr_request.post_bad_ip):
            connection_info = {
                u'vpn-interface-name': u'Tunnel%d' % tunnel_id,
                u'ipsec-policy-id': u'%d' % ipsec_policy_id,
                u'local-device': {u'ip-address': u'%s/24' % conflicting_ip,
                                  u'tunnel-ip-address': u'10.10.10.10'},
                u'remote-device': {u'tunnel-ip-address': u'10.10.10.20'}
            }
            self.csr.create_ipsec_connection(connection_info)
            self.addCleanup(self._remove_resource_for_test,
                            self.csr.delete_ipsec_connection,
                            'Tunnel%d' % tunnel_id)
            self.assertEqual(requests.codes.BAD_REQUEST, self.csr.status)

    def test_create_ipsec_connection_with_max_mtu(self):
        """Create an IPSec connection with max MTU value."""
        tunnel_id, ipsec_policy_id = self._prepare_for_site_conn_create()
        with httmock.HTTMock(csr_request.token, csr_request.post,
                             csr_request.get_mtu):
            connection_info = {
                u'vpn-interface-name': u'Tunnel%d' % tunnel_id,
                u'ipsec-policy-id': u'%d' % ipsec_policy_id,
                u'mtu': 9192,
                u'local-device': {u'ip-address': u'10.3.0.1/24',
                                  u'tunnel-ip-address': u'10.10.10.10'},
                u'remote-device': {u'tunnel-ip-address': u'10.10.10.20'}
            }
            location = self.csr.create_ipsec_connection(connection_info)
            self.addCleanup(self._remove_resource_for_test,
                            self.csr.delete_ipsec_connection,
                            'Tunnel%d' % tunnel_id)
            self.assertEqual(requests.codes.CREATED, self.csr.status)
            self.assertIn('vpn-svc/site-to-site/Tunnel%d' % tunnel_id,
                          location)
            # Check the hard-coded items that get set as well...
            content = self.csr.get_request(location, full_url=True)
            self.assertEqual(requests.codes.OK, self.csr.status)
            expected_connection = {u'kind': u'object#vpn-site-to-site',
                                   u'ike-profile-id': None,
                                   u'ip-version': u'ipv4'}
            expected_connection.update(connection_info)
            self.assertEqual(expected_connection, content)

    def test_create_ipsec_connection_with_bad_mtu(self):
        """Negative test of connection create with unsupported MTU value."""
        tunnel_id, ipsec_policy_id = self._prepare_for_site_conn_create()
        with httmock.HTTMock(csr_request.token, csr_request.post_bad_mtu):
            connection_info = {
                u'vpn-interface-name': u'Tunnel%d' % tunnel_id,
                u'ipsec-policy-id': u'%d' % ipsec_policy_id,
                u'mtu': 9193,
                u'local-device': {u'ip-address': u'10.3.0.1/24',
                                  u'tunnel-ip-address': u'10.10.10.10'},
                u'remote-device': {u'tunnel-ip-address': u'10.10.10.20'}
            }
            self.csr.create_ipsec_connection(connection_info)
            self.addCleanup(self._remove_resource_for_test,
                            self.csr.delete_ipsec_connection,
                            'Tunnel%d' % tunnel_id)
            self.assertEqual(requests.codes.BAD_REQUEST, self.csr.status)

    def test_status_when_no_tunnels_exist(self):
        """Get status, when there are no tunnels."""
        with httmock.HTTMock(csr_request.token, csr_request.get_none):
            tunnels = self.csr.read_tunnel_statuses()
            self.assertEqual(requests.codes.OK, self.csr.status)
            self.assertEqual([], tunnels)

    def test_status_for_one_tunnel(self):
        """Get status of one tunnel."""
        # Create the IPsec site-to-site connection first
        tunnel_id, ipsec_policy_id = self._prepare_for_site_conn_create()
        tunnel_id = 123  # Must hard code to work with mock
        with httmock.HTTMock(csr_request.token, csr_request.post,
                             csr_request.normal_get):
            connection_info = {
                u'vpn-interface-name': u'Tunnel123',
                u'ipsec-policy-id': u'%d' % ipsec_policy_id,
                u'local-device': {u'ip-address': u'10.3.0.1/24',
                                  u'tunnel-ip-address': u'10.10.10.10'},
                u'remote-device': {u'tunnel-ip-address': u'10.10.10.20'}
            }
            location = self.csr.create_ipsec_connection(connection_info)
            self.addCleanup(self._remove_resource_for_test,
                            self.csr.delete_ipsec_connection,
                            u'Tunnel123')
            self.assertEqual(requests.codes.CREATED, self.csr.status)
            self.assertIn('vpn-svc/site-to-site/Tunnel%d' % tunnel_id,
                          location)
        with httmock.HTTMock(csr_request.token, csr_request.normal_get):
            tunnels = self.csr.read_tunnel_statuses()
            self.assertEqual(requests.codes.OK, self.csr.status)
            self.assertEqual([(u'Tunnel123', u'DOWN-NEGOTIATING'), ], tunnels)


class TestCsrRestIkeKeepaliveCreate(base.BaseTestCase):

    """Test IKE keepalive REST requests.

    This is a global configuration that will apply to all VPN tunnels and
    is used to specify Dead Peer Detection information. Currently, the API
    supports DELETE API, but a bug has been created to remove the API and
    add an indicator of when the capability is disabled.
    """

    def setUp(self):
        super(TestCsrRestIkeKeepaliveCreate, self).setUp()
        self.csr = csr_client.CsrRestClient('localhost', '10.10.10.10',
                                            'stack', 'cisco')

    def test_configure_ike_keepalive(self):
        """Set IKE keep-alive (aka Dead Peer Detection) for the CSR."""
        with httmock.HTTMock(csr_request.token, csr_request.put,
                             csr_request.normal_get):
            keepalive_info = {'interval': 60, 'retry': 4}
            self.csr.configure_ike_keepalive(keepalive_info)
            self.assertEqual(requests.codes.NO_CONTENT, self.csr.status)
            content = self.csr.get_request('vpn-svc/ike/keepalive')
            self.assertEqual(requests.codes.OK, self.csr.status)
            expected = {'periodic': False}
            expected.update(keepalive_info)
            self.assertDictContainsSubset(expected, content)

    def test_disable_ike_keepalive(self):
        """Disable IKE keep-alive (aka Dead Peer Detection) for the CSR."""
        with httmock.HTTMock(csr_request.token, csr_request.delete,
                             csr_request.put, csr_request.get_not_configured):
            if csr_request.FIXED_CSCum10324:
                # TODO(pcm) Is this how to disable?
                keepalive_info = {'interval': 0, 'retry': 4}
                self.csr.configure_ike_keepalive(keepalive_info)
                self.assertEqual(requests.codes.NO_CONTENT, self.csr.status)
            else:
                self.csr.delete_request('vnc-svc/ike/keepalive')
                self.assertIn(self.csr.status,
                              (requests.codes.NO_CONTENT,
                               requests.codes.NOT_FOUND))
                self.csr.get_request('vpn-svc/ike/keepalive')
                self.assertEqual(requests.codes.NOT_FOUND, self.csr.status)


class TestCsrRestStaticRoute(base.BaseTestCase):

    """Test static route REST requests.

    A static route is added for the peer's private network. Would create
    a route for each of the peer CIDRs specified for the VPN connection.
    """

    def setUp(self):
        super(TestCsrRestStaticRoute, self).setUp()
        self.csr = csr_client.CsrRestClient('localhost', '10.10.10.10',
                                            'stack', 'cisco')

    def test_create_delete_static_route(self):
        """Create and then delete a static route for the tunnel."""
        cidr = u'10.1.0.0/24'
        interface = u'GigabitEthernet1'
        expected_id = '10.1.0.0_24_GigabitEthernet1'
        with httmock.HTTMock(csr_request.token, csr_request.post,
                             csr_request.normal_get):
            route_info = {u'destination-network': cidr,
                          u'outgoing-interface': interface}
            location = self.csr.create_static_route(route_info)
            self.assertEqual(requests.codes.CREATED, self.csr.status)
            self.assertIn('routing-svc/static-routes/%s' % expected_id,
                          location)
            # Check the hard-coded items that get set as well...
            content = self.csr.get_request(location, full_url=True)
            self.assertEqual(requests.codes.OK, self.csr.status)
            expected_route = {u'kind': u'object#static-route',
                              u'next-hop-router': None,
                              u'admin-distance': 1}
            expected_route.update(route_info)
            self.assertEqual(expected_route, content)
        # Now delete and verify that static route is gone
        with httmock.HTTMock(csr_request.token, csr_request.delete,
                             csr_request.no_such_resource):
            route_id = csr_client.make_route_id(cidr, interface)
            self.csr.delete_static_route(route_id)
            self.assertEqual(requests.codes.NO_CONTENT, self.csr.status)
            content = self.csr.get_request(location, full_url=True)
            self.assertEqual(requests.codes.NOT_FOUND, self.csr.status)



# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# Copyright 2013 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

"""ml2_initial

Revision ID: 5ac71e65402c
Revises: 128e042a2b68
Create Date: 2013-05-27 16:08:40.853821

"""

# revision identifiers, used by Alembic.
revision = '5ac71e65402c'
down_revision = '128e042a2b68'

# Change to ['*'] if this migration applies to all plugins

migration_for_plugins = [
    'neutron.plugins.ml2.plugin.Ml2Plugin'
]

from alembic import op
import sqlalchemy as sa


from neutron.db import migration


def upgrade(active_plugins=None, options=None):
    if not migration.should_run(active_plugins, migration_for_plugins):
        return

    ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        'ml2_network_segments',
        sa.Column('id', sa.String(length=36), nullable=False),
        sa.Column('network_id', sa.String(length=36), nullable=False),
        sa.Column('network_type', sa.String(length=32), nullable=False),
        sa.Column('physical_network', sa.String(length=64), nullable=True),
        sa.Column('segmentation_id', sa.Integer(), nullable=True),
        sa.ForeignKeyConstraint(['network_id'], ['networks.id'],
                                ondelete='CASCADE'),
        sa.PrimaryKeyConstraint('id')
    )
    op.create_table(
        'ml2_vlan_allocations',
        sa.Column('physical_network', sa.String(length=64), nullable=False),
        sa.Column('vlan_id', sa.Integer(), autoincrement=False,
                  nullable=False),
        sa.Column('allocated', sa.Boolean(), autoincrement=False,
                  nullable=False),
        sa.PrimaryKeyConstraint('physical_network', 'vlan_id')
    )
    op.create_table(
        'ml2_flat_allocations',
        sa.Column('physical_network', sa.String(length=64), nullable=False),
        sa.PrimaryKeyConstraint('physical_network')
    )
    ### end Alembic commands ###


def downgrade(active_plugins=None, options=None):
    if not migration.should_run(active_plugins, migration_for_plugins):
        return

    ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('ml2_network_segments')
    op.drop_table('ml2_flat_allocations')
    op.drop_table('ml2_vlan_allocations')
    ### end Alembic commands ###



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2013 VMware, Inc
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Unit tests for extension extended attribute
"""

from oslo.config import cfg
import webob.exc as webexc

import neutron
from neutron.api import extensions
from neutron.api.v2 import attributes
from neutron.common import config
from neutron import manager
from neutron.plugins.common import constants
from neutron.plugins.openvswitch import ovs_neutron_plugin
from neutron import quota
from neutron.tests import base
from neutron.tests.unit.extensions import extendedattribute as extattr
from neutron.tests.unit import test_api_v2
from neutron.tests.unit import testlib_api
from neutron import wsgi

_uuid = test_api_v2._uuid
_get_path = test_api_v2._get_path
extensions_path = ':'.join(neutron.tests.unit.extensions.__path__)


class ExtensionExtendedAttributeTestPlugin(
    ovs_neutron_plugin.OVSNeutronPluginV2):

    supported_extension_aliases = [
        'ext-obj-test', "extended-ext-attr"
    ]

    def __init__(self, configfile=None):
        super(ExtensionExtendedAttributeTestPlugin, self)
        self.objs = []
        self.objh = {}

    def create_ext_test_resource(self, context, ext_test_resource):
        obj = ext_test_resource['ext_test_resource']
        id = _uuid()
        obj['id'] = id
        self.objs.append(obj)
        self.objh.update({id: obj})
        return obj

    def get_ext_test_resources(self, context, filters=None, fields=None):
        return self.objs

    def get_ext_test_resource(self, context, id, fields=None):
        return self.objh[id]


class ExtensionExtendedAttributeTestCase(base.BaseTestCase):
    def setUp(self):
        super(ExtensionExtendedAttributeTestCase, self).setUp()
        plugin = (
            "neutron.tests.unit.test_extension_extended_attribute."
            "ExtensionExtendedAttributeTestPlugin"
        )

        # point config file to: neutron/tests/etc/neutron.conf.test
        args = ['--config-file', test_api_v2.etcdir('neutron.conf.test')]
        config.parse(args=args)

        self.setup_coreplugin(plugin)

        ext_mgr = extensions.PluginAwareExtensionManager(
            extensions_path,
            {constants.CORE: ExtensionExtendedAttributeTestPlugin}
        )
        ext_mgr.extend_resources("2.0", {})
        extensions.PluginAwareExtensionManager._instance = ext_mgr

        app = config.load_paste_app('extensions_test_app')
        self._api = extensions.ExtensionMiddleware(app, ext_mgr=ext_mgr)

        self._tenant_id = "8c70909f-b081-452d-872b-df48e6c355d1"
        # Save the global RESOURCE_ATTRIBUTE_MAP
        self.saved_attr_map = {}
        for resource, attrs in attributes.RESOURCE_ATTRIBUTE_MAP.iteritems():
            self.saved_attr_map[resource] = attrs.copy()
        # Add the resources to the global attribute map
        # This is done here as the setup process won't
        # initialize the main API router which extends
        # the global attribute map
        attributes.RESOURCE_ATTRIBUTE_MAP.update(
            extattr.EXTENDED_ATTRIBUTES_2_0)
        self.agentscheduler_dbMinxin = manager.NeutronManager.get_plugin()
        self.addCleanup(self.restore_attribute_map)

        quota.QUOTAS._driver = None
        cfg.CONF.set_override('quota_driver', 'neutron.quota.ConfDriver',
                              group='QUOTAS')

    def restore_attribute_map(self):
        # Restore the original RESOURCE_ATTRIBUTE_MAP
        attributes.RESOURCE_ATTRIBUTE_MAP = self.saved_attr_map

    def _do_request(self, method, path, data=None, params=None, action=None):
        content_type = 'application/json'
        body = None
        if data is not None:  # empty dict is valid
            body = wsgi.Serializer().serialize(data, content_type)

        req = testlib_api.create_request(
            path, body, content_type,
            method, query_string=params)
        res = req.get_response(self._api)
        if res.status_code >= 400:
            raise webexc.HTTPClientError(detail=res.body, code=res.status_code)
        if res.status_code != webexc.HTTPNoContent.code:
            return res.json

    def _ext_test_resource_create(self, attr=None):
        data = {
            "ext_test_resource": {
                "tenant_id": self._tenant_id,
                "name": "test",
                extattr.EXTENDED_ATTRIBUTE: attr
            }
        }

        res = self._do_request('POST', _get_path('ext_test_resources'), data)
        return res['ext_test_resource']

    def test_ext_test_resource_create(self):
        ext_test_resource = self._ext_test_resource_create()
        attr = _uuid()
        ext_test_resource = self._ext_test_resource_create(attr)
        self.assertEqual(ext_test_resource[extattr.EXTENDED_ATTRIBUTE], attr)

    def test_ext_test_resource_get(self):
        attr = _uuid()
        obj = self._ext_test_resource_create(attr)
        obj_id = obj['id']
        res = self._do_request('GET', _get_path(
            'ext_test_resources/{0}'.format(obj_id)))
        obj2 = res['ext_test_resource']
        self.assertEqual(obj2[extattr.EXTENDED_ATTRIBUTE], attr)



# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# Copyright 2013 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

"""nsx_mappings

Revision ID: 50e86cb2637a
Revises: havana
Create Date: 2013-10-26 14:37:30.012149

"""

# revision identifiers, used by Alembic.
revision = '50e86cb2637a'
down_revision = '1fcfc149aca4'

# Change to ['*'] if this migration applies to all plugins

migration_for_plugins = [
    'neutron.plugins.nicira.NeutronPlugin.NvpPluginV2',
    'neutron.plugins.nicira.NeutronServicePlugin.NvpAdvancedPlugin',
    'neutron.plugins.vmware.plugin.NsxPlugin',
    'neutron.plugins.vmware.plugin.NsxServicePlugin'
]

from alembic import op
import sqlalchemy as sa

from neutron.db import migration


def upgrade(active_plugins=None, options=None):
    if not migration.should_run(active_plugins, migration_for_plugins):
        return

    op.create_table('neutron_nsx_port_mappings',
                    sa.Column('neutron_id', sa.String(length=36),
                              nullable=False),
                    sa.Column('nsx_port_id', sa.String(length=36),
                              nullable=False),
                    sa.Column('nsx_switch_id', sa.String(length=36),
                              nullable=True),
                    sa.ForeignKeyConstraint(['neutron_id'], ['ports.id'],
                                            ondelete='CASCADE'),
                    sa.PrimaryKeyConstraint('neutron_id'))

    op.execute("INSERT INTO neutron_nsx_port_mappings SELECT quantum_id as "
               "neutron_id, nvp_id as nsx_port_id, null as nsx_switch_id from"
               " quantum_nvp_port_mapping")
    op.drop_table('quantum_nvp_port_mapping')


def downgrade(active_plugins=None, options=None):
    if not migration.should_run(active_plugins, migration_for_plugins):
        return

    # Restore table to pre-icehouse version
    op.create_table('quantum_nvp_port_mapping',
                    sa.Column('quantum_id', sa.String(length=36),
                              nullable=False),
                    sa.Column('nvp_id', sa.String(length=36),
                              nullable=False),
                    sa.ForeignKeyConstraint(['quantum_id'], ['ports.id'],
                                            ondelete='CASCADE'),
                    sa.PrimaryKeyConstraint('quantum_id'))
    op.execute("INSERT INTO quantum_nvp_port_mapping SELECT neutron_id as "
               "quantum_id, nsx_port_id as nvp_id from"
               " neutron_nsx_port_mappings")
    op.drop_table('neutron_nsx_port_mappings')



# Copyright 2014 OneConvergence, Inc. All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# @author: Kedar Kulkarni, One Convergence, Inc.

"""NVSD agent code for security group events."""

import socket
import time

import eventlet

from neutron.agent.linux import ovs_lib
from neutron.agent import rpc as agent_rpc
from neutron.agent import securitygroups_rpc as sg_rpc
from neutron.common import config as logging_config
from neutron.common import topics
from neutron import context as n_context
from neutron.extensions import securitygroup as ext_sg
from neutron.openstack.common import log as logging
from neutron.openstack.common.rpc import dispatcher
from neutron.openstack.common.rpc import proxy
from neutron.plugins.oneconvergence.lib import config

LOG = logging.getLogger(__name__)


class NVSDAgentRpcCallback(object):

    RPC_API_VERSION = '1.0'

    def __init__(self, context, agent, sg_agent):
        self.context = context
        self.agent = agent
        self.sg_agent = sg_agent

    def port_update(self, context, **kwargs):
        LOG.debug(_("port_update received: %s"), kwargs)
        port = kwargs.get('port')
        # Validate that port is on OVS
        vif_port = self.agent.int_br.get_vif_port_by_id(port['id'])
        if not vif_port:
            return

        if ext_sg.SECURITYGROUPS in port:
            self.sg_agent.refresh_firewall()


class SecurityGroupServerRpcApi(proxy.RpcProxy,
                                sg_rpc.SecurityGroupServerRpcApiMixin):
    def __init__(self, topic):
        super(SecurityGroupServerRpcApi, self).__init__(
            topic=topic, default_version=sg_rpc.SG_RPC_VERSION)


class SecurityGroupAgentRpcCallback(
    sg_rpc.SecurityGroupAgentRpcCallbackMixin):

    RPC_API_VERSION = sg_rpc.SG_RPC_VERSION

    def __init__(self, context, sg_agent):
        self.context = context
        self.sg_agent = sg_agent


class SecurityGroupAgentRpc(sg_rpc.SecurityGroupAgentRpcMixin):

    def __init__(self, context, root_helper):
        self.context = context

        self.plugin_rpc = SecurityGroupServerRpcApi(topics.PLUGIN)
        self.root_helper = root_helper
        self.init_firewall()


class NVSDNeutronAgent(object):
    # history
    #   1.0 Initial version
    #   1.1 Support Security Group RPC
    RPC_API_VERSION = '1.1'

    def __init__(self, integ_br, root_helper, polling_interval):

        self.int_br = ovs_lib.OVSBridge(integ_br, root_helper)
        self.polling_interval = polling_interval
        self.root_helper = root_helper
        self.setup_rpc()
        self.ports = set()

    def setup_rpc(self):

        self.host = socket.gethostname()
        self.agent_id = 'nvsd-q-agent.%s' % self.host
        LOG.info(_("RPC agent_id: %s"), self.agent_id)

        self.topic = topics.AGENT
        self.context = n_context.get_admin_context_without_session()
        self.sg_agent = SecurityGroupAgentRpc(self.context,
                                              self.root_helper)

        # RPC network init
        # Handle updates from service
        self.callback_oc = NVSDAgentRpcCallback(self.context,
                                                self, self.sg_agent)
        self.callback_sg = SecurityGroupAgentRpcCallback(self.context,
                                                         self.sg_agent)
        self.dispatcher = dispatcher.RpcDispatcher([self.callback_oc,
                                                    self.callback_sg])
        # Define the listening consumer for the agent
        consumers = [[topics.PORT, topics.UPDATE],
                     [topics.SECURITY_GROUP, topics.UPDATE]]
        self.connection = agent_rpc.create_consumers(self.dispatcher,
                                                     self.topic,
                                                     consumers)

    def _update_ports(self, registered_ports):
        ports = self.int_br.get_vif_port_set()
        if ports == registered_ports:
            return
        added = ports - registered_ports
        removed = registered_ports - ports
        return {'current': ports,
                'added': added,
                'removed': removed}

    def _process_devices_filter(self, port_info):
        if 'added' in port_info:
            self.sg_agent.prepare_devices_filter(port_info['added'])
        if 'removed' in port_info:
            self.sg_agent.remove_devices_filter(port_info['removed'])

    def daemon_loop(self):
        """Main processing loop for OC Plugin Agent."""

        ports = set()
        while True:
            try:
                port_info = self._update_ports(ports)
                if port_info:
                    LOG.debug(_("Port list is updated"))
                    self._process_devices_filter(port_info)
                    ports = port_info['current']
                    self.ports = ports
            except Exception:
                LOG.exception(_("Error in agent event loop"))

            LOG.debug(_("AGENT looping....."))
            time.sleep(self.polling_interval)


def main():
    eventlet.monkey_patch()
    config.CONF(project='neutron')
    logging_config.setup_logging(config.CONF)

    integ_br = config.AGENT.integration_bridge
    root_helper = config.AGENT.root_helper
    polling_interval = config.AGENT.polling_interval
    agent = NVSDNeutronAgent(integ_br, root_helper, polling_interval)
    LOG.info(_("NVSD Agent initialized successfully, now running... "))

    # Start everything.
    agent.daemon_loop()



# Copyright 2012 VMware, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sqlalchemy as sa
from sqlalchemy import orm
from sqlalchemy.orm import exc
from sqlalchemy.orm import scoped_session

from neutron.api.v2 import attributes as attr
from neutron.common import constants
from neutron.db import db_base_plugin_v2
from neutron.db import model_base
from neutron.db import models_v2
from neutron.extensions import securitygroup as ext_sg
from neutron.openstack.common import uuidutils


IP_PROTOCOL_MAP = {constants.PROTO_NAME_TCP: constants.PROTO_NUM_TCP,
                   constants.PROTO_NAME_UDP: constants.PROTO_NUM_UDP,
                   constants.PROTO_NAME_ICMP: constants.PROTO_NUM_ICMP,
                   constants.PROTO_NAME_ICMP_V6: constants.PROTO_NUM_ICMP_V6}


class SecurityGroup(model_base.BASEV2, models_v2.HasId, models_v2.HasTenant):
    """Represents a v2 neutron security group."""

    name = sa.Column(sa.String(255))
    description = sa.Column(sa.String(255))


class SecurityGroupPortBinding(model_base.BASEV2):
    """Represents binding between neutron ports and security profiles."""

    port_id = sa.Column(sa.String(36),
                        sa.ForeignKey("ports.id",
                                      ondelete='CASCADE'),
                        primary_key=True)
    security_group_id = sa.Column(sa.String(36),
                                  sa.ForeignKey("securitygroups.id"),
                                  primary_key=True)

    # Add a relationship to the Port model in order to instruct SQLAlchemy to
    # eagerly load security group bindings
    ports = orm.relationship(
        models_v2.Port,
        backref=orm.backref("security_groups",
                            lazy='joined', cascade='delete'))


class SecurityGroupRule(model_base.BASEV2, models_v2.HasId,
                        models_v2.HasTenant):
    """Represents a v2 neutron security group rule."""

    security_group_id = sa.Column(sa.String(36),
                                  sa.ForeignKey("securitygroups.id",
                                                ondelete="CASCADE"),
                                  nullable=False)

    remote_group_id = sa.Column(sa.String(36),
                                sa.ForeignKey("securitygroups.id",
                                              ondelete="CASCADE"),
                                nullable=True)

    direction = sa.Column(sa.Enum('ingress', 'egress',
                                  name='securitygrouprules_direction'))
    ethertype = sa.Column(sa.String(40))
    protocol = sa.Column(sa.String(40))
    port_range_min = sa.Column(sa.Integer)
    port_range_max = sa.Column(sa.Integer)
    remote_ip_prefix = sa.Column(sa.String(255))
    security_group = orm.relationship(
        SecurityGroup,
        backref=orm.backref('rules', cascade='all,delete'),
        primaryjoin="SecurityGroup.id==SecurityGroupRule.security_group_id")
    source_group = orm.relationship(
        SecurityGroup,
        backref=orm.backref('source_rules', cascade='all,delete'),
        primaryjoin="SecurityGroup.id==SecurityGroupRule.remote_group_id")


class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase):
    """Mixin class to add security group to db_plugin_base_v2."""

    __native_bulk_support = True

    def create_security_group_bulk(self, context, security_group_rule):
        return self._create_bulk('security_group', context,
                                 security_group_rule)

    def create_security_group(self, context, security_group, default_sg=False):
        """Create security group.

        If default_sg is true that means we are a default security group for
        a given tenant if it does not exist.
        """
        s = security_group['security_group']
        tenant_id = self._get_tenant_id_for_create(context, s)

        if not default_sg:
            self._ensure_default_security_group(context, tenant_id)

        with context.session.begin(subtransactions=True):
            security_group_db = SecurityGroup(id=s.get('id') or (
                                              uuidutils.generate_uuid()),
                                              description=s['description'],
                                              tenant_id=tenant_id,
                                              name=s['name'])
            context.session.add(security_group_db)
            for ethertype in ext_sg.sg_supported_ethertypes:
                if s.get('name') == 'default':
                    # Allow intercommunication
                    ingress_rule = SecurityGroupRule(
                        id=uuidutils.generate_uuid(), tenant_id=tenant_id,
                        security_group=security_group_db,
                        direction='ingress',
                        ethertype=ethertype,
                        source_group=security_group_db)
                    context.session.add(ingress_rule)

                egress_rule = SecurityGroupRule(
                    id=uuidutils.generate_uuid(), tenant_id=tenant_id,
                    security_group=security_group_db,
                    direction='egress',
                    ethertype=ethertype)
                context.session.add(egress_rule)

        return self._make_security_group_dict(security_group_db)

    def get_security_groups(self, context, filters=None, fields=None,
                            sorts=None, limit=None,
                            marker=None, page_reverse=False, default_sg=False):

        # If default_sg is True do not call _ensure_default_security_group()
        # so this can be done recursively. Context.tenant_id is checked
        # because all the unit tests do not explicitly set the context on
        # GETS. TODO(arosen)  context handling can probably be improved here.
        if not default_sg and context.tenant_id:
            self._ensure_default_security_group(context, context.tenant_id)
        marker_obj = self._get_marker_obj(context, 'security_group', limit,
                                          marker)
        return self._get_collection(context,
                                    SecurityGroup,
                                    self._make_security_group_dict,
                                    filters=filters, fields=fields,
                                    sorts=sorts,
                                    limit=limit, marker_obj=marker_obj,
                                    page_reverse=page_reverse)

    def get_security_groups_count(self, context, filters=None):
        return self._get_collection_count(context, SecurityGroup,
                                          filters=filters)

    def get_security_group(self, context, id, fields=None, tenant_id=None):
        """Tenant id is given to handle the case when creating a security
        group rule on behalf of another use.
        """

        if tenant_id:
            tmp_context_tenant_id = context.tenant_id
            context.tenant_id = tenant_id

        try:
            with context.session.begin(subtransactions=True):
                ret = self._make_security_group_dict(self._get_security_group(
                                                     context, id), fields)
                ret['security_group_rules'] = self.get_security_group_rules(
                    context, {'security_group_id': [id]})
        finally:
            if tenant_id:
                context.tenant_id = tmp_context_tenant_id
        return ret

    def _get_security_group(self, context, id):
        try:
            query = self._model_query(context, SecurityGroup)
            sg = query.filter(SecurityGroup.id == id).one()

        except exc.NoResultFound:
            raise ext_sg.SecurityGroupNotFound(id=id)
        return sg

    def delete_security_group(self, context, id):
        filters = {'security_group_id': [id]}
        ports = self._get_port_security_group_bindings(context, filters)
        if ports:
            raise ext_sg.SecurityGroupInUse(id=id)
        # confirm security group exists
        sg = self._get_security_group(context, id)

        if sg['name'] == 'default' and not context.is_admin:
            raise ext_sg.SecurityGroupCannotRemoveDefault()
        with context.session.begin(subtransactions=True):
            context.session.delete(sg)

    def update_security_group(self, context, id, security_group):
        s = security_group['security_group']
        with context.session.begin(subtransactions=True):
            sg = self._get_security_group(context, id)
            if sg['name'] == 'default' and 'name' in s:
                raise ext_sg.SecurityGroupCannotUpdateDefault()
            sg.update(s)
        return self._make_security_group_dict(sg)

    def _make_security_group_dict(self, security_group, fields=None):
        res = {'id': security_group['id'],
               'name': security_group['name'],
               'tenant_id': security_group['tenant_id'],
               'description': security_group['description']}
        res['security_group_rules'] = [self._make_security_group_rule_dict(r)
                                       for r in security_group.rules]
        return self._fields(res, fields)

    def _make_security_group_binding_dict(self, security_group, fields=None):
        res = {'port_id': security_group['port_id'],
               'security_group_id': security_group['security_group_id']}
        return self._fields(res, fields)

    def _create_port_security_group_binding(self, context, port_id,
                                            security_group_id):
        with context.session.begin(subtransactions=True):
            db = SecurityGroupPortBinding(port_id=port_id,
                                          security_group_id=security_group_id)
            context.session.add(db)

    def _get_port_security_group_bindings(self, context,
                                          filters=None, fields=None):
        return self._get_collection(context,
                                    SecurityGroupPortBinding,
                                    self._make_security_group_binding_dict,
                                    filters=filters, fields=fields)

    def _delete_port_security_group_bindings(self, context, port_id):
        query = self._model_query(context, SecurityGroupPortBinding)
        bindings = query.filter(
            SecurityGroupPortBinding.port_id == port_id)
        with context.session.begin(subtransactions=True):
            for binding in bindings:
                context.session.delete(binding)

    def create_security_group_rule_bulk(self, context, security_group_rule):
        return self._create_bulk('security_group_rule', context,
                                 security_group_rule)

    def create_security_group_rule_bulk_native(self, context,
                                               security_group_rule):
        r = security_group_rule['security_group_rules']

        scoped_session(context.session)
        security_group_id = self._validate_security_group_rules(
            context, security_group_rule)
        with context.session.begin(subtransactions=True):
            if not self.get_security_group(context, security_group_id):
                raise ext_sg.SecurityGroupNotFound(id=security_group_id)

            self._check_for_duplicate_rules(context, r)
            ret = []
            for rule_dict in r:
                rule = rule_dict['security_group_rule']
                tenant_id = self._get_tenant_id_for_create(context, rule)
                db = SecurityGroupRule(
                    id=uuidutils.generate_uuid(), tenant_id=tenant_id,
                    security_group_id=rule['security_group_id'],
                    direction=rule['direction'],
                    remote_group_id=rule.get('remote_group_id'),
                    ethertype=rule['ethertype'],
                    protocol=rule['protocol'],
                    port_range_min=rule['port_range_min'],
                    port_range_max=rule['port_range_max'],
                    remote_ip_prefix=rule.get('remote_ip_prefix'))
                context.session.add(db)
            ret.append(self._make_security_group_rule_dict(db))
        return ret

    def create_security_group_rule(self, context, security_group_rule):
        bulk_rule = {'security_group_rules': [security_group_rule]}
        return self.create_security_group_rule_bulk_native(context,
                                                           bulk_rule)[0]

    def _get_ip_proto_number(self, protocol):
        if protocol is None:
            return
        return IP_PROTOCOL_MAP.get(protocol, protocol)

    def _validate_port_range(self, rule):
        """Check that port_range is valid."""
        if (rule['port_range_min'] is None and
            rule['port_range_max'] is None):
            return
        if not rule['protocol']:
            raise ext_sg.SecurityGroupProtocolRequiredWithPorts()
        ip_proto = self._get_ip_proto_number(rule['protocol'])
        if ip_proto in [constants.PROTO_NUM_TCP, constants.PROTO_NUM_UDP]:
            if (rule['port_range_min'] is not None and
                rule['port_range_min'] <= rule['port_range_max']):
                pass
            else:
                raise ext_sg.SecurityGroupInvalidPortRange()
        elif ip_proto == constants.PROTO_NUM_ICMP:
            for attr, field in [('port_range_min', 'type'),
                                ('port_range_max', 'code')]:
                if rule[attr] > 255:
                    raise ext_sg.SecurityGroupInvalidIcmpValue(
                        field=field, attr=attr, value=rule[attr])

    def _validate_security_group_rules(self, context, security_group_rule):
        """Check that rules being installed.

        Check that all rules belong to the same security
        group, remote_group_id/security_group_id belong to the same tenant,
        and rules are valid.
        """
        new_rules = set()
        tenant_ids = set()
        for rules in security_group_rule['security_group_rules']:
            rule = rules.get('security_group_rule')
            new_rules.add(rule['security_group_id'])

            self._validate_port_range(rule)

            if rule['remote_ip_prefix'] and rule['remote_group_id']:
                raise ext_sg.SecurityGroupRemoteGroupAndRemoteIpPrefix()

            if rule['tenant_id'] not in tenant_ids:
                tenant_ids.add(rule['tenant_id'])
            remote_group_id = rule.get('remote_group_id')
            # Check that remote_group_id exists for tenant
            if remote_group_id:
                self.get_security_group(context, remote_group_id,
                                        tenant_id=rule['tenant_id'])
        if len(new_rules) > 1:
            raise ext_sg.SecurityGroupNotSingleGroupRules()
        security_group_id = new_rules.pop()

        # Confirm single tenant and that the tenant has permission
        # to add rules to this security group.
        if len(tenant_ids) > 1:
            raise ext_sg.SecurityGroupRulesNotSingleTenant()
        for tenant_id in tenant_ids:
            self.get_security_group(context, security_group_id,
                                    tenant_id=tenant_id)
        return security_group_id

    def _make_security_group_rule_dict(self, security_group_rule, fields=None):
        res = {'id': security_group_rule['id'],
               'tenant_id': security_group_rule['tenant_id'],
               'security_group_id': security_group_rule['security_group_id'],
               'ethertype': security_group_rule['ethertype'],
               'direction': security_group_rule['direction'],
               'protocol': security_group_rule['protocol'],
               'port_range_min': security_group_rule['port_range_min'],
               'port_range_max': security_group_rule['port_range_max'],
               'remote_ip_prefix': security_group_rule['remote_ip_prefix'],
               'remote_group_id': security_group_rule['remote_group_id']}

        return self._fields(res, fields)

    def _make_security_group_rule_filter_dict(self, security_group_rule):
        sgr = security_group_rule['security_group_rule']
        res = {'tenant_id': [sgr['tenant_id']],
               'security_group_id': [sgr['security_group_id']],
               'direction': [sgr['direction']]}

        include_if_present = ['protocol', 'port_range_max', 'port_range_min',
                              'ethertype', 'remote_ip_prefix',
                              'remote_group_id']
        for key in include_if_present:
            value = sgr.get(key)
            if value:
                res[key] = [value]
        return res

    def _check_for_duplicate_rules(self, context, security_group_rules):
        for i in security_group_rules:
            found_self = False
            for j in security_group_rules:
                if i['security_group_rule'] == j['security_group_rule']:
                    if found_self:
                        raise ext_sg.DuplicateSecurityGroupRuleInPost(rule=i)
                    found_self = True

            # Check in database if rule exists
            filters = self._make_security_group_rule_filter_dict(i)
            db_rules = self.get_security_group_rules(context, filters)
            # Note(arosen): the call to get_security_group_rules wildcards
            # values in the filter that have a value of [None]. For
            # example, filters = {'remote_group_id': [None]} will return
            # all security group rules regardless of their value of
            # remote_group_id. Therefore it is not possible to do this
            # query unless the behavior of _get_collection()
            # is changed which cannot be because other methods are already
            # relying on this behavor. Therefore, we do the filtering
            # below to check for these corner cases.
            for db_rule in db_rules:
                # need to remove id from db_rule for matching
                id = db_rule.pop('id')
                if (i['security_group_rule'] == db_rule):
                    raise ext_sg.SecurityGroupRuleExists(id=id)

    def get_security_group_rules(self, context, filters=None, fields=None,
                                 sorts=None, limit=None, marker=None,
                                 page_reverse=False):
        marker_obj = self._get_marker_obj(context, 'security_group_rule',
                                          limit, marker)
        return self._get_collection(context,
                                    SecurityGroupRule,
                                    self._make_security_group_rule_dict,
                                    filters=filters, fields=fields,
                                    sorts=sorts,
                                    limit=limit, marker_obj=marker_obj,
                                    page_reverse=page_reverse)

    def get_security_group_rules_count(self, context, filters=None):
        return self._get_collection_count(context, SecurityGroupRule,
                                          filters=filters)

    def get_security_group_rule(self, context, id, fields=None):
        security_group_rule = self._get_security_group_rule(context, id)
        return self._make_security_group_rule_dict(security_group_rule, fields)

    def _get_security_group_rule(self, context, id):
        try:
            query = self._model_query(context, SecurityGroupRule)
            sgr = query.filter(SecurityGroupRule.id == id).one()
        except exc.NoResultFound:
            raise ext_sg.SecurityGroupRuleNotFound(id=id)
        return sgr

    def delete_security_group_rule(self, context, id):
        with context.session.begin(subtransactions=True):
            rule = self._get_security_group_rule(context, id)
            context.session.delete(rule)

    def _extend_port_dict_security_group(self, port_res, port_db):
        # Security group bindings will be retrieved from the sqlalchemy
        # model. As they're loaded eagerly with ports because of the
        # joined load they will not cause an extra query.
        security_group_ids = [sec_group_mapping['security_group_id'] for
                              sec_group_mapping in port_db.security_groups]
        port_res[ext_sg.SECURITYGROUPS] = security_group_ids
        return port_res

    # Register dict extend functions for ports
    db_base_plugin_v2.NeutronDbPluginV2.register_dict_extend_funcs(
        attr.PORTS, ['_extend_port_dict_security_group'])

    def _process_port_create_security_group(self, context, port,
                                            security_group_ids):
        if attr.is_attr_set(security_group_ids):
            for security_group_id in security_group_ids:
                self._create_port_security_group_binding(context, port['id'],
                                                         security_group_id)
        # Convert to list as a set might be passed here and
        # this has to be serialized
        port[ext_sg.SECURITYGROUPS] = (security_group_ids and
                                       list(security_group_ids) or [])

    def _ensure_default_security_group(self, context, tenant_id):
        """Create a default security group if one doesn't exist.

        :returns: the default security group id.
        """
        filters = {'name': ['default'], 'tenant_id': [tenant_id]}
        default_group = self.get_security_groups(context, filters,
                                                 default_sg=True)
        if not default_group:
            security_group = {'security_group': {'name': 'default',
                                                 'tenant_id': tenant_id,
                                                 'description': 'default'}}
            ret = self.create_security_group(context, security_group, True)
            return ret['id']
        else:
            return default_group[0]['id']

    def _get_security_groups_on_port(self, context, port):
        """Check that all security groups on port belong to tenant.

        :returns: all security groups IDs on port belonging to tenant.
        """
        p = port['port']
        if not attr.is_attr_set(p.get(ext_sg.SECURITYGROUPS)):
            return
        if p.get('device_owner') and p['device_owner'].startswith('network:'):
            return

        valid_groups = self.get_security_groups(context, fields=['id'])
        valid_group_map = dict((g['id'], g['id']) for g in valid_groups)
        try:
            return set([valid_group_map[sg_id]
                        for sg_id in p.get(ext_sg.SECURITYGROUPS, [])])
        except KeyError as e:
            raise ext_sg.SecurityGroupNotFound(id=str(e))

    def _ensure_default_security_group_on_port(self, context, port):
        # we don't apply security groups for dhcp, router
        if (port['port'].get('device_owner') and
                port['port']['device_owner'].startswith('network:')):
            return
        tenant_id = self._get_tenant_id_for_create(context,
                                                   port['port'])
        default_sg = self._ensure_default_security_group(context, tenant_id)
        if attr.is_attr_set(port['port'].get(ext_sg.SECURITYGROUPS)):
            sgids = port['port'].get(ext_sg.SECURITYGROUPS)
        else:
            sgids = [default_sg]
        port['port'][ext_sg.SECURITYGROUPS] = sgids

    def _check_update_deletes_security_groups(self, port):
        """Return True if port has as a security group and it's value
        is either [] or not is_attr_set, otherwise return False
        """
        if (ext_sg.SECURITYGROUPS in port['port'] and
            not (attr.is_attr_set(port['port'][ext_sg.SECURITYGROUPS])
                 and port['port'][ext_sg.SECURITYGROUPS] != [])):
            return True
        return False

    def _check_update_has_security_groups(self, port):
        """Return True if port has as a security group and False if the
        security_group field is is_attr_set or [].
        """
        if (ext_sg.SECURITYGROUPS in port['port'] and
            (attr.is_attr_set(port['port'][ext_sg.SECURITYGROUPS]) and
             port['port'][ext_sg.SECURITYGROUPS] != [])):
            return True
        return False



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (C) 2013 Midokura PTE LTD
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# @author: Ryu Ishimoto, Midokura Japan KK

import mock
import uuid


def get_bridge_mock(id=None, **kwargs):
    if id is None:
        id = str(uuid.uuid4())

    bridge = mock.Mock()
    bridge.get_id.return_value = id
    bridge.get_tenant_id.return_value = kwargs.get("tenant_id", "test-tenant")
    bridge.get_name.return_value = kwargs.get("name", "net")
    bridge.get_ports.return_value = []
    bridge.get_peer_ports.return_value = []
    bridge.get_admin_state_up.return_value = kwargs.get("admin_state_up", True)
    return bridge


def get_bridge_port_mock(id=None, bridge_id=None, **kwargs):
    if id is None:
        id = str(uuid.uuid4())
    if bridge_id is None:
        bridge_id = str(uuid.uuid4())

    port = mock.Mock()
    port.get_id.return_value = id
    port.get_bridge_id.return_value = bridge_id
    port.get_admin_state_up.return_value = kwargs.get("admin_state_up", True)
    port.get_type.return_value = "Bridge"
    port.create.return_value = port
    return port


def get_chain_mock(id=None, tenant_id='test-tenant', name='chain',
                   rules=None):
    if id is None:
        id = str(uuid.uuid4())

    if rules is None:
        rules = []

    chain = mock.Mock()
    chain.get_id.return_value = id
    chain.get_tenant_id.return_value = tenant_id
    chain.get_name.return_value = name
    chain.get_rules.return_value = rules
    return chain


def get_port_group_mock(id=None, tenant_id='test-tenant', name='pg'):
    if id is None:
        id = str(uuid.uuid4())

    port_group = mock.Mock()
    port_group.get_id.return_value = id
    port_group.get_tenant_id.return_value = tenant_id
    port_group.get_name.return_value = name
    return port_group


def get_router_mock(id=None, **kwargs):
    if id is None:
        id = str(uuid.uuid4())

    router = mock.Mock()
    router.get_id.return_value = id
    router.get_tenant_id.return_value = kwargs.get("tenant_id", "test-tenant")
    router.get_name.return_value = kwargs.get("name", "router")
    router.get_ports.return_value = []
    router.get_peer_ports.return_value = []
    router.get_routes.return_value = []
    router.get_admin_state_up.return_value = kwargs.get("admin_state_up", True)
    return router


def get_rule_mock(id=None, chain_id=None, properties=None):
    if id is None:
        id = str(uuid.uuid4())

    if chain_id is None:
        chain_id = str(uuid.uuid4())

    if properties is None:
        properties = {}

    rule = mock.Mock()
    rule.get_id.return_value = id
    rule.get_chain_id.return_value = chain_id
    rule.get_properties.return_value = properties
    return rule


def get_subnet_mock(bridge_id=None, gateway_ip='10.0.0.1',
                    subnet_prefix='10.0.0.0', subnet_len=int(24)):
    if bridge_id is None:
        bridge_id = str(uuid.uuid4())

    subnet = mock.Mock()
    subnet.get_id.return_value = subnet_prefix + '/' + str(subnet_len)
    subnet.get_bridge_id.return_value = bridge_id
    subnet.get_default_gateway.return_value = gateway_ip
    subnet.get_subnet_prefix.return_value = subnet_prefix
    subnet.get_subnet_length.return_value = subnet_len
    return subnet


class MidonetLibMockConfig():

    def __init__(self, inst):
        self.inst = inst

    def _create_bridge(self, **kwargs):
        return get_bridge_mock(**kwargs)

    def _create_router(self, **kwargs):
        return get_router_mock(**kwargs)

    def _create_subnet(self, bridge, gateway_ip, subnet_prefix, subnet_len):
        return get_subnet_mock(bridge.get_id(), gateway_ip=gateway_ip,
                               subnet_prefix=subnet_prefix,
                               subnet_len=subnet_len)

    def _add_bridge_port(self, bridge, **kwargs):
        return get_bridge_port_mock(bridge_id=bridge.get_id(), **kwargs)

    def _get_bridge(self, id):
        return get_bridge_mock(id=id)

    def _get_port(self, id):
        return get_bridge_port_mock(id=id)

    def _get_router(self, id):
        return get_router_mock(id=id)

    def _update_bridge(self, id, **kwargs):
        return get_bridge_mock(id=id, **kwargs)

    def setup(self):
        # Bridge methods side effects
        self.inst.create_bridge.side_effect = self._create_bridge
        self.inst.get_bridge.side_effect = self._get_bridge
        self.inst.update_bridge.side_effect = self._update_bridge

        # Subnet methods side effects
        self.inst.create_subnet.side_effect = self._create_subnet

        # Port methods side effects
        ex_bp = self.inst.add_bridge_port
        ex_bp.side_effect = self._add_bridge_port
        self.inst.get_port.side_effect = self._get_port

        # Router methods side effects
        self.inst.create_router.side_effect = self._create_router
        self.inst.get_router.side_effect = self._get_router


class MidoClientMockConfig():

    def __init__(self, inst):
        self.inst = inst
        self.chains_in = None
        self.port_groups_in = None
        self.chains_out = None
        self.rules_out = None
        self.port_groups_out = None

    def _get_query_tenant_id(self, query):
        if query is not None and query['tenant_id']:
            tenant_id = query['tenant_id']
        else:
            tenant_id = 'test-tenant'
        return tenant_id

    def _get_bridge(self, id):
        return get_bridge_mock(id=id)

    def _get_chain(self, id, query=None):
        if not self.chains_in:
            return []

        tenant_id = self._get_query_tenant_id(query)
        for chain in self.chains_in:
            chain_id = chain['id']
            if chain_id is id:
                rule_mocks = []
                if 'rules' in chain:
                    for rule in chain['rules']:
                        rule_mocks.append(
                            get_rule_mock(id=rule['id'],
                                          chain_id=id,
                                          properties=rule['properties']))

                return get_chain_mock(id=chain_id, name=chain['name'],
                                      tenant_id=tenant_id, rules=rule_mocks)
        return None

    def _get_chains(self, query=None):
        if not self.chains_in:
            return []

        tenant_id = self._get_query_tenant_id(query)
        self.chains_out = []
        self.rules_out = []
        for chain in self.chains_in:
            chain_id = chain['id']

            rule_mocks = []
            if 'rules' in chain:
                for rule in chain['rules']:
                    rule_mocks.append(
                        get_rule_mock(id=rule['id'],
                                      chain_id=id,
                                      properties=rule['properties']))
                    self.rules_out += rule_mocks

            self.chains_out.append(get_chain_mock(id=chain_id,
                                                  name=chain['name'],
                                                  tenant_id=tenant_id,
                                                  rules=rule_mocks))
        return self.chains_out

    def _get_port_groups(self, query=None):
        if not self.port_groups_in:
            return []

        tenant_id = self._get_query_tenant_id(query)
        self.port_groups_out = []
        for port_group in self.port_groups_in:
            self.port_groups_out.append(get_port_group_mock(
                id=port_group['id'], name=port_group['name'],
                tenant_id=tenant_id))
        return self.port_groups_out

    def _get_router(self, id):
        return get_router_mock(id=id)

    def _add_bridge_port(self, bridge):
        return get_bridge_port_mock(bridge_id=bridge.get_id())

    def setup(self):
        self.inst.get_bridge.side_effect = self._get_bridge
        self.inst.get_chains.side_effect = self._get_chains
        self.inst.get_chain.side_effect = self._get_chain
        self.inst.get_port_groups.side_effect = self._get_port_groups
        self.inst.get_router.side_effect = self._get_router
        self.inst.add_bridge_port.side_effect = self._add_bridge_port



# vim: tabstop=4 shiftwidth=4 softtabstop=4
#
# Copyright 2013 New Dream Network, LLC (DreamHost)
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# @author: Mark McClain, DreamHost

import contextlib

import mock

from neutron.plugins.common import constants
from neutron.services.loadbalancer.agent import agent_manager as manager
from neutron.tests import base


class TestManager(base.BaseTestCase):
    def setUp(self):
        super(TestManager, self).setUp()

        mock_conf = mock.Mock()
        mock_conf.device_driver = ['devdriver']

        self.mock_importer = mock.patch.object(manager, 'importutils').start()

        rpc_mock_cls = mock.patch(
            'neutron.services.loadbalancer.agent.agent_api.LbaasAgentApi'
        ).start()

        # disable setting up periodic state reporting
        mock_conf.AGENT.report_interval = 0

        self.mgr = manager.LbaasAgentManager(mock_conf)
        self.rpc_mock = rpc_mock_cls.return_value
        self.log = mock.patch.object(manager, 'LOG').start()
        self.driver_mock = mock.Mock()
        self.mgr.device_drivers = {'devdriver': self.driver_mock}
        self.mgr.instance_mapping = {'1': 'devdriver', '2': 'devdriver'}
        self.mgr.needs_resync = False

    def test_initialize_service_hook(self):
        with mock.patch.object(self.mgr, 'sync_state') as sync:
            self.mgr.initialize_service_hook(mock.Mock())
            sync.assert_called_once_with()

    def test_periodic_resync_needs_sync(self):
        with mock.patch.object(self.mgr, 'sync_state') as sync:
            self.mgr.needs_resync = True
            self.mgr.periodic_resync(mock.Mock())
            sync.assert_called_once_with()

    def test_periodic_resync_no_sync(self):
        with mock.patch.object(self.mgr, 'sync_state') as sync:
            self.mgr.needs_resync = False
            self.mgr.periodic_resync(mock.Mock())
            self.assertFalse(sync.called)

    def test_collect_stats(self):
        self.mgr.collect_stats(mock.Mock())
        self.rpc_mock.update_pool_stats.assert_has_calls([
            mock.call('1', mock.ANY),
            mock.call('2', mock.ANY)
        ])

    def test_collect_stats_exception(self):
        self.driver_mock.get_stats.side_effect = Exception

        self.mgr.collect_stats(mock.Mock())

        self.assertFalse(self.rpc_mock.called)
        self.assertTrue(self.mgr.needs_resync)
        self.assertTrue(self.log.exception.called)

    def _sync_state_helper(self, ready, reloaded, destroyed):
        with contextlib.nested(
            mock.patch.object(self.mgr, '_reload_pool'),
            mock.patch.object(self.mgr, '_destroy_pool')
        ) as (reload, destroy):

            self.rpc_mock.get_ready_devices.return_value = ready

            self.mgr.sync_state()

            self.assertEqual(len(reloaded), len(reload.mock_calls))
            self.assertEqual(len(destroyed), len(destroy.mock_calls))

            reload.assert_has_calls([mock.call(i) for i in reloaded])
            destroy.assert_has_calls([mock.call(i) for i in destroyed])
            self.assertFalse(self.mgr.needs_resync)

    def test_sync_state_all_known(self):
        self._sync_state_helper(['1', '2'], ['1', '2'], [])

    def test_sync_state_all_unknown(self):
        self.mgr.instance_mapping = {}
        self._sync_state_helper(['1', '2'], ['1', '2'], [])

    def test_sync_state_destroy_all(self):
        self._sync_state_helper([], [], ['1', '2'])

    def test_sync_state_both(self):
        self.mgr.instance_mapping = {'1': 'devdriver'}
        self._sync_state_helper(['2'], ['2'], ['1'])

    def test_sync_state_exception(self):
        self.rpc_mock.get_ready_devices.side_effect = Exception

        self.mgr.sync_state()

        self.assertTrue(self.log.exception.called)
        self.assertTrue(self.mgr.needs_resync)

    def test_reload_pool(self):
        config = {'driver': 'devdriver'}
        self.rpc_mock.get_logical_device.return_value = config
        pool_id = 'new_id'
        self.assertNotIn(pool_id, self.mgr.instance_mapping)

        self.mgr._reload_pool(pool_id)

        self.driver_mock.deploy_instance.assert_called_once_with(config)
        self.assertIn(pool_id, self.mgr.instance_mapping)
        self.rpc_mock.pool_deployed.assert_called_once_with(pool_id)

    def test_reload_pool_driver_not_found(self):
        config = {'driver': 'unknown_driver'}
        self.rpc_mock.get_logical_device.return_value = config
        pool_id = 'new_id'
        self.assertNotIn(pool_id, self.mgr.instance_mapping)

        self.mgr._reload_pool(pool_id)

        self.assertTrue(self.log.error.called)
        self.assertFalse(self.driver_mock.deploy_instance.called)
        self.assertNotIn(pool_id, self.mgr.instance_mapping)
        self.assertFalse(self.rpc_mock.pool_deployed.called)

    def test_reload_pool_exception_on_driver(self):
        config = {'driver': 'devdriver'}
        self.rpc_mock.get_logical_device.return_value = config
        self.driver_mock.deploy_instance.side_effect = Exception
        pool_id = 'new_id'
        self.assertNotIn(pool_id, self.mgr.instance_mapping)

        self.mgr._reload_pool(pool_id)

        self.driver_mock.deploy_instance.assert_called_once_with(config)
        self.assertNotIn(pool_id, self.mgr.instance_mapping)
        self.assertFalse(self.rpc_mock.pool_deployed.called)
        self.assertTrue(self.log.exception.called)
        self.assertTrue(self.mgr.needs_resync)

    def test_destroy_pool(self):
        pool_id = '1'
        self.assertIn(pool_id, self.mgr.instance_mapping)

        self.mgr._destroy_pool(pool_id)

        self.driver_mock.undeploy_instance.assert_called_once_with(pool_id)
        self.assertNotIn(pool_id, self.mgr.instance_mapping)
        self.rpc_mock.pool_destroyed.assert_called_once_with(pool_id)
        self.assertFalse(self.mgr.needs_resync)

    def test_destroy_pool_exception_on_driver(self):
        pool_id = '1'
        self.assertIn(pool_id, self.mgr.instance_mapping)
        self.driver_mock.undeploy_instance.side_effect = Exception

        self.mgr._destroy_pool(pool_id)

        self.driver_mock.undeploy_instance.assert_called_once_with(pool_id)
        self.assertIn(pool_id, self.mgr.instance_mapping)
        self.assertFalse(self.rpc_mock.pool_destroyed.called)
        self.assertTrue(self.log.exception.called)
        self.assertTrue(self.mgr.needs_resync)

    def test_get_driver_unknown_device(self):
        self.assertRaises(manager.DeviceNotFoundOnAgent,
                          self.mgr._get_driver, 'unknown')

    def test_remove_orphans(self):
        self.mgr.remove_orphans()
        self.driver_mock.remove_orphans.assert_called_once_with(['1', '2'])

    def test_create_vip(self):
        vip = {'id': 'id1', 'pool_id': '1'}
        self.mgr.create_vip(mock.Mock(), vip)
        self.driver_mock.create_vip.assert_called_once_with(vip)
        self.rpc_mock.update_status.assert_called_once_with('vip', vip['id'],
                                                            constants.ACTIVE)

    def test_create_vip_failed(self):
        vip = {'id': 'id1', 'pool_id': '1'}
        self.driver_mock.create_vip.side_effect = Exception
        self.mgr.create_vip(mock.Mock(), vip)
        self.driver_mock.create_vip.assert_called_once_with(vip)
        self.rpc_mock.update_status.assert_called_once_with('vip', vip['id'],
                                                            constants.ERROR)

    def test_update_vip(self):
        old_vip = {'id': 'id1'}
        vip = {'id': 'id1', 'pool_id': '1'}
        self.mgr.update_vip(mock.Mock(), old_vip, vip)
        self.driver_mock.update_vip.assert_called_once_with(old_vip, vip)
        self.rpc_mock.update_status.assert_called_once_with('vip', vip['id'],
                                                            constants.ACTIVE)

    def test_update_vip_failed(self):
        old_vip = {'id': 'id1'}
        vip = {'id': 'id1', 'pool_id': '1'}
        self.driver_mock.update_vip.side_effect = Exception
        self.mgr.update_vip(mock.Mock(), old_vip, vip)
        self.driver_mock.update_vip.assert_called_once_with(old_vip, vip)
        self.rpc_mock.update_status.assert_called_once_with('vip', vip['id'],
                                                            constants.ERROR)

    def test_delete_vip(self):
        vip = {'id': 'id1', 'pool_id': '1'}
        self.mgr.delete_vip(mock.Mock(), vip)
        self.driver_mock.delete_vip.assert_called_once_with(vip)

    def test_create_pool(self):
        pool = {'id': 'id1'}
        self.assertNotIn(pool['id'], self.mgr.instance_mapping)
        self.mgr.create_pool(mock.Mock(), pool, 'devdriver')
        self.driver_mock.create_pool.assert_called_once_with(pool)
        self.rpc_mock.update_status.assert_called_once_with('pool', pool['id'],
                                                            constants.ACTIVE)
        self.assertIn(pool['id'], self.mgr.instance_mapping)

    def test_create_pool_failed(self):
        pool = {'id': 'id1'}
        self.assertNotIn(pool['id'], self.mgr.instance_mapping)
        self.driver_mock.create_pool.side_effect = Exception
        self.mgr.create_pool(mock.Mock(), pool, 'devdriver')
        self.driver_mock.create_pool.assert_called_once_with(pool)
        self.rpc_mock.update_status.assert_called_once_with('pool', pool['id'],
                                                            constants.ERROR)
        self.assertNotIn(pool['id'], self.mgr.instance_mapping)

    def test_update_pool(self):
        old_pool = {'id': '1'}
        pool = {'id': '1'}
        self.mgr.update_pool(mock.Mock(), old_pool, pool)
        self.driver_mock.update_pool.assert_called_once_with(old_pool, pool)
        self.rpc_mock.update_status.assert_called_once_with('pool', pool['id'],
                                                            constants.ACTIVE)

    def test_update_pool_failed(self):
        old_pool = {'id': '1'}
        pool = {'id': '1'}
        self.driver_mock.update_pool.side_effect = Exception
        self.mgr.update_pool(mock.Mock(), old_pool, pool)
        self.driver_mock.update_pool.assert_called_once_with(old_pool, pool)
        self.rpc_mock.update_status.assert_called_once_with('pool', pool['id'],
                                                            constants.ERROR)

    def test_delete_pool(self):
        pool = {'id': '1'}
        self.assertIn(pool['id'], self.mgr.instance_mapping)
        self.mgr.delete_pool(mock.Mock(), pool)
        self.driver_mock.delete_pool.assert_called_once_with(pool)
        self.assertNotIn(pool['id'], self.mgr.instance_mapping)

    def test_create_member(self):
        member = {'id': 'id1', 'pool_id': '1'}
        self.mgr.create_member(mock.Mock(), member)
        self.driver_mock.create_member.assert_called_once_with(member)
        self.rpc_mock.update_status.assert_called_once_with('member',
                                                            member['id'],
                                                            constants.ACTIVE)

    def test_create_member_failed(self):
        member = {'id': 'id1', 'pool_id': '1'}
        self.driver_mock.create_member.side_effect = Exception
        self.mgr.create_member(mock.Mock(), member)
        self.driver_mock.create_member.assert_called_once_with(member)
        self.rpc_mock.update_status.assert_called_once_with('member',
                                                            member['id'],
                                                            constants.ERROR)

    def test_update_member(self):
        old_member = {'id': 'id1'}
        member = {'id': 'id1', 'pool_id': '1'}
        self.mgr.update_member(mock.Mock(), old_member, member)
        self.driver_mock.update_member.assert_called_once_with(old_member,
                                                               member)
        self.rpc_mock.update_status.assert_called_once_with('member',
                                                            member['id'],
                                                            constants.ACTIVE)

    def test_update_member_failed(self):
        old_member = {'id': 'id1'}
        member = {'id': 'id1', 'pool_id': '1'}
        self.driver_mock.update_member.side_effect = Exception
        self.mgr.update_member(mock.Mock(), old_member, member)
        self.driver_mock.update_member.assert_called_once_with(old_member,
                                                               member)
        self.rpc_mock.update_status.assert_called_once_with('member',
                                                            member['id'],
                                                            constants.ERROR)

    def test_delete_member(self):
        member = {'id': 'id1', 'pool_id': '1'}
        self.mgr.delete_member(mock.Mock(), member)
        self.driver_mock.delete_member.assert_called_once_with(member)

    def test_create_monitor(self):
        monitor = {'id': 'id1'}
        assoc_id = {'monitor_id': monitor['id'], 'pool_id': '1'}
        self.mgr.create_pool_health_monitor(mock.Mock(), monitor, '1')
        self.driver_mock.create_pool_health_monitor.assert_called_once_with(
            monitor, '1')
        self.rpc_mock.update_status.assert_called_once_with('health_monitor',
                                                            assoc_id,
                                                            constants.ACTIVE)

    def test_create_monitor_failed(self):
        monitor = {'id': 'id1'}
        assoc_id = {'monitor_id': monitor['id'], 'pool_id': '1'}
        self.driver_mock.create_pool_health_monitor.side_effect = Exception
        self.mgr.create_pool_health_monitor(mock.Mock(), monitor, '1')
        self.driver_mock.create_pool_health_monitor.assert_called_once_with(
            monitor, '1')
        self.rpc_mock.update_status.assert_called_once_with('health_monitor',
                                                            assoc_id,
                                                            constants.ERROR)

    def test_update_monitor(self):
        monitor = {'id': 'id1'}
        assoc_id = {'monitor_id': monitor['id'], 'pool_id': '1'}
        self.mgr.update_pool_health_monitor(mock.Mock(), monitor, monitor, '1')
        self.driver_mock.update_pool_health_monitor.assert_called_once_with(
            monitor, monitor, '1')
        self.rpc_mock.update_status.assert_called_once_with('health_monitor',
                                                            assoc_id,
                                                            constants.ACTIVE)

    def test_update_monitor_failed(self):
        monitor = {'id': 'id1'}
        assoc_id = {'monitor_id': monitor['id'], 'pool_id': '1'}
        self.driver_mock.update_pool_health_monitor.side_effect = Exception
        self.mgr.update_pool_health_monitor(mock.Mock(), monitor, monitor, '1')
        self.driver_mock.update_pool_health_monitor.assert_called_once_with(
            monitor, monitor, '1')
        self.rpc_mock.update_status.assert_called_once_with('health_monitor',
                                                            assoc_id,
                                                            constants.ERROR)

    def test_delete_monitor(self):
        monitor = {'id': 'id1'}
        self.mgr.delete_pool_health_monitor(mock.Mock(), monitor, '1')
        self.driver_mock.delete_pool_health_monitor.assert_called_once_with(
            monitor, '1')

    def test_agent_disabled(self):
        payload = {'admin_state_up': False}
        self.mgr.agent_updated(mock.Mock(), payload)
        self.driver_mock.undeploy_instance.assert_has_calls(
            [mock.call('1'), mock.call('2')])



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2011 Cisco Systems, Inc.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# @author: Ying Liu, Cisco Systems, Inc.
#

from webob import exc

from neutron.api import api_common as common
from neutron.api import extensions
from neutron.manager import NeutronManager
from neutron.plugins.cisco.common import cisco_exceptions as exception
from neutron.plugins.cisco.common import cisco_faults as faults
from neutron.plugins.cisco.extensions import _qos_view as qos_view
from neutron import wsgi


class Qos(extensions.ExtensionDescriptor):
    """Qos extension file."""

    @classmethod
    def get_name(cls):
        """Returns Ext Resource Name."""
        return "Cisco qos"

    @classmethod
    def get_alias(cls):
        """Returns Ext Resource Alias."""
        return "Cisco qos"

    @classmethod
    def get_description(cls):
        """Returns Ext Resource Description."""
        return "qos includes qos_name and qos_desc"

    @classmethod
    def get_namespace(cls):
        """Returns Ext Resource Namespace."""
        return "http://docs.ciscocloud.com/api/ext/qos/v1.0"

    @classmethod
    def get_updated(cls):
        """Returns Ext Resource update."""
        return "2011-07-25T13:25:27-06:00"

    @classmethod
    def get_resources(cls):
        """Returns Ext Resources."""
        parent_resource = dict(member_name="tenant",
                               collection_name="extensions/csco/tenants")

        controller = QosController(NeutronManager.get_plugin())
        return [extensions.ResourceExtension('qoss', controller,
                                             parent=parent_resource)]


class QosController(common.NeutronController, wsgi.Controller):
    """qos API controller based on NeutronController."""

    _qos_ops_param_list = [
        {'param-name': 'qos_name', 'required': True},
        {'param-name': 'qos_desc', 'required': True},
    ]

    _serialization_metadata = {
        "application/xml": {
            "attributes": {
                "qos": ["id", "name"],
            },
        },
    }

    def __init__(self, plugin):
        self._resource_name = 'qos'
        self._plugin = plugin

    def index(self, request, tenant_id):
        """Returns a list of qos ids."""
        return self._items(request, tenant_id, is_detail=False)

    def _items(self, request, tenant_id, is_detail):
        """Returns a list of qoss."""
        qoss = self._plugin.get_all_qoss(tenant_id)
        builder = qos_view.get_view_builder(request)
        result = [builder.build(qos, is_detail)['qos'] for qos in qoss]
        return dict(qoss=result)

    # pylint: disable-msg=E1101
    def show(self, request, tenant_id, id):
        """Returns qos details for the given qos id."""
        try:
            qos = self._plugin.get_qos_details(tenant_id, id)
            builder = qos_view.get_view_builder(request)
            #build response with details
            result = builder.build(qos, True)
            return dict(qoss=result)
        except exception.QosNotFound as exp:
            return faults.Fault(faults.QosNotFound(exp))

    def create(self, request, tenant_id):
        """Creates a new qos for a given tenant."""
        #look for qos name in request
        try:
            body = self._deserialize(request.body, request.get_content_type())
            req_body = self._prepare_request_body(body,
                                                  self._qos_ops_param_list)
            req_params = req_body[self._resource_name]
        except exc.HTTPError as exp:
            return faults.Fault(exp)
        qos = self._plugin.create_qos(tenant_id,
                                      req_params['qos_name'],
                                      req_params['qos_desc'])
        builder = qos_view.get_view_builder(request)
        result = builder.build(qos)
        return dict(qoss=result)

    def update(self, request, tenant_id, id):
        """Updates the name for the qos with the given id."""
        try:
            body = self._deserialize(request.body, request.get_content_type())
            req_body = self._prepare_request_body(body,
                                                  self._qos_ops_param_list)
            req_params = req_body[self._resource_name]
        except exc.HTTPError as exp:
            return faults.Fault(exp)
        try:
            qos = self._plugin.rename_qos(tenant_id, id,
                                          req_params['qos_name'])

            builder = qos_view.get_view_builder(request)
            result = builder.build(qos, True)
            return dict(qoss=result)
        except exception.QosNotFound as exp:
            return faults.Fault(faults.QosNotFound(exp))

    def delete(self, request, tenant_id, id):
        """Destroys the qos with the given id."""
        try:
            self._plugin.delete_qos(tenant_id, id)
            return exc.HTTPOk()
        except exception.QosNotFound as exp:
            return faults.Fault(faults.QosNotFound(exp))



# Copyright 2014 Big Switch Networks, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
# @author: Kevin Benton, kevin.benton@bigswitch.com
#
import httplib
import socket

from contextlib import nested
import mock
from oslo.config import cfg

from neutron.manager import NeutronManager
from neutron.plugins.bigswitch import servermanager
from neutron.tests.unit.bigswitch import test_restproxy_plugin as test_rp

HTTPCON = 'httplib.HTTPConnection'
SERVERMANAGER = 'neutron.plugins.bigswitch.servermanager'


class ServerManagerTests(test_rp.BigSwitchProxyPluginV2TestCase):

    def test_no_servers(self):
        cfg.CONF.set_override('servers', [], 'RESTPROXY')
        self.assertRaises(cfg.Error, servermanager.ServerPool)

    def test_malformed_servers(self):
        cfg.CONF.set_override('servers', ['a:b:c'], 'RESTPROXY')
        self.assertRaises(cfg.Error, servermanager.ServerPool)

    def test_sticky_cert_fetch_fail(self):
        pl = NeutronManager.get_plugin()
        pl.servers.ssl = True
        with mock.patch(
            'ssl.get_server_certificate',
            side_effect=Exception('There is no more entropy in the universe')
        ) as sslgetmock:
            self.assertRaises(
                cfg.Error,
                pl.servers._get_combined_cert_for_server,
                *('example.org', 443)
            )
            sslgetmock.assert_has_calls([mock.call(('example.org', 443))])

    def test_consistency_watchdog(self):
        pl = NeutronManager.get_plugin()
        pl.servers.capabilities = []
        self.watch_p.stop()
        with nested(
            mock.patch('eventlet.sleep'),
            mock.patch(
                SERVERMANAGER + '.ServerPool.rest_call',
                side_effect=servermanager.RemoteRestError(
                    reason='Failure to break loop'
                )
            )
        ) as (smock, rmock):
            # should return immediately without consistency capability
            pl.servers._consistency_watchdog()
            self.assertFalse(smock.called)
            pl.servers.capabilities = ['consistency']
            self.assertRaises(servermanager.RemoteRestError,
                              pl.servers._consistency_watchdog)

    def test_file_put_contents(self):
        pl = NeutronManager.get_plugin()
        with mock.patch(SERVERMANAGER + '.open', create=True) as omock:
            pl.servers._file_put_contents('somepath', 'contents')
            omock.assert_has_calls([mock.call('somepath', 'w')])
            omock.return_value.__enter__.return_value.assert_has_calls([
                mock.call.write('contents')
            ])

    def test_combine_certs_to_file(self):
        pl = NeutronManager.get_plugin()
        with mock.patch(SERVERMANAGER + '.open', create=True) as omock:
            omock.return_value.__enter__().read.return_value = 'certdata'
            pl.servers._combine_certs_to_file(['cert1.pem', 'cert2.pem'],
                                              'combined.pem')
            # mock shared between read and write file handles so the calls
            # are mixed together
            omock.assert_has_calls([
                mock.call('combined.pem', 'w'),
                mock.call('cert1.pem', 'r'),
                mock.call('cert2.pem', 'r'),
            ], any_order=True)
            omock.return_value.__enter__.return_value.assert_has_calls([
                mock.call.read(),
                mock.call.write('certdata'),
                mock.call.read(),
                mock.call.write('certdata')
            ])

    def test_reconnect_cached_connection(self):
        sp = servermanager.ServerPool()
        with mock.patch(HTTPCON) as conmock:
            rv = conmock.return_value
            rv.getresponse.return_value.getheader.return_value = 'HASH'
            sp.servers[0].capabilities = ['keep-alive']
            sp.servers[0].rest_call('GET', '/first')
            # raise an error on re-use to verify reconnect
            # return okay the second time so the reconnect works
            rv.request.side_effect = [httplib.ImproperConnectionState(),
                                      mock.MagicMock()]
            sp.servers[0].rest_call('GET', '/second')
        uris = [c[1][1] for c in rv.request.mock_calls]
        expected = [
            sp.base_uri + '/first',
            sp.base_uri + '/second',
            sp.base_uri + '/second',
        ]
        self.assertEqual(uris, expected)

    def test_no_reconnect_recurse_to_infinity(self):
        # retry uses recursion when a reconnect is necessary
        # this test makes sure it stops after 1 recursive call
        sp = servermanager.ServerPool()
        with mock.patch(HTTPCON) as conmock:
            rv = conmock.return_value
            # hash header must be string instead of mock object
            rv.getresponse.return_value.getheader.return_value = 'HASH'
            sp.servers[0].capabilities = ['keep-alive']
            sp.servers[0].rest_call('GET', '/first')
            # after retrying once, the rest call should raise the
            # exception up
            rv.request.side_effect = httplib.ImproperConnectionState()
            self.assertRaises(httplib.ImproperConnectionState,
                              sp.servers[0].rest_call,
                              *('GET', '/second'))
            # 1 for the first call, 2 for the second with retry
            self.assertEqual(rv.request.call_count, 3)

    def test_socket_error(self):
        sp = servermanager.ServerPool()
        with mock.patch(HTTPCON) as conmock:
            conmock.return_value.request.side_effect = socket.timeout()
            resp = sp.servers[0].rest_call('GET', '/')
            self.assertEqual(resp, (0, None, None, None))



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2013 Cloudbase Solutions SRL
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
# @author: Alessandro Pilotti, Cloudbase Solutions Srl

from six.moves import xrange
from sqlalchemy.orm import exc

from neutron.common import exceptions as n_exc
import neutron.db.api as db_api
from neutron.db import models_v2
from neutron.openstack.common import log as logging
from neutron.plugins.hyperv.common import constants
from neutron.plugins.hyperv import model as hyperv_model

LOG = logging.getLogger(__name__)


class HyperVPluginDB(object):
    def initialize(self):
        db_api.configure_db()

    def reserve_vlan(self, session):
        with session.begin(subtransactions=True):
            alloc_q = session.query(hyperv_model.VlanAllocation)
            alloc_q = alloc_q.filter_by(allocated=False)
            alloc = alloc_q.first()
            if alloc:
                LOG.debug(_("Reserving vlan %(vlan_id)s on physical network "
                            "%(physical_network)s from pool"),
                          {'vlan_id': alloc.vlan_id,
                           'physical_network': alloc.physical_network})
                alloc.allocated = True
                return (alloc.physical_network, alloc.vlan_id)
        raise n_exc.NoNetworkAvailable()

    def reserve_flat_net(self, session):
        with session.begin(subtransactions=True):
            alloc_q = session.query(hyperv_model.VlanAllocation)
            alloc_q = alloc_q.filter_by(allocated=False,
                                        vlan_id=constants.FLAT_VLAN_ID)
            alloc = alloc_q.first()
            if alloc:
                LOG.debug(_("Reserving flat physical network "
                            "%(physical_network)s from pool"),
                          {'physical_network': alloc.physical_network})
                alloc.allocated = True
                return alloc.physical_network
        raise n_exc.NoNetworkAvailable()

    def reserve_specific_vlan(self, session, physical_network, vlan_id):
        with session.begin(subtransactions=True):
            try:
                alloc_q = session.query(hyperv_model.VlanAllocation)
                alloc_q = alloc_q.filter_by(
                    physical_network=physical_network,
                    vlan_id=vlan_id)
                alloc = alloc_q.one()
                if alloc.allocated:
                    if vlan_id == constants.FLAT_VLAN_ID:
                        raise n_exc.FlatNetworkInUse(
                            physical_network=physical_network)
                    else:
                        raise n_exc.VlanIdInUse(
                            vlan_id=vlan_id,
                            physical_network=physical_network)
                LOG.debug(_("Reserving specific vlan %(vlan_id)s on physical "
                            "network %(physical_network)s from pool"),
                          {'vlan_id': vlan_id,
                           'physical_network': physical_network})
                alloc.allocated = True
            except exc.NoResultFound:
                raise n_exc.NoNetworkAvailable()

    def reserve_specific_flat_net(self, session, physical_network):
        return self.reserve_specific_vlan(session, physical_network,
                                          constants.FLAT_VLAN_ID)

    def add_network_binding(self, session, network_id, network_type,
                            physical_network, segmentation_id):
        with session.begin(subtransactions=True):
            binding = hyperv_model.NetworkBinding(
                network_id, network_type,
                physical_network,
                segmentation_id)
            session.add(binding)

    def get_port(self, port_id):
        session = db_api.get_session()
        try:
            port = session.query(models_v2.Port).filter_by(id=port_id).one()
        except exc.NoResultFound:
            port = None
        return port

    def get_network_binding(self, session, network_id):
        session = session or db_api.get_session()
        try:
            binding_q = session.query(hyperv_model.NetworkBinding)
            binding_q = binding_q.filter_by(network_id=network_id)
            return binding_q.one()
        except exc.NoResultFound:
            return

    def set_port_status(self, port_id, status):
        session = db_api.get_session()
        try:
            port = session.query(models_v2.Port).filter_by(id=port_id).one()
            port['status'] = status
            session.merge(port)
            session.flush()
        except exc.NoResultFound:
            raise n_exc.PortNotFound(port_id=port_id)

    def release_vlan(self, session, physical_network, vlan_id):
        with session.begin(subtransactions=True):
            try:
                alloc_q = session.query(hyperv_model.VlanAllocation)
                alloc_q = alloc_q.filter_by(physical_network=physical_network,
                                            vlan_id=vlan_id)
                alloc = alloc_q.one()
                alloc.allocated = False
                #session.delete(alloc)
                LOG.debug(_("Releasing vlan %(vlan_id)s on physical network "
                            "%(physical_network)s"),
                          {'vlan_id': vlan_id,
                           'physical_network': physical_network})
            except exc.NoResultFound:
                LOG.warning(_("vlan_id %(vlan_id)s on physical network "
                              "%(physical_network)s not found"),
                            {'vlan_id': vlan_id,
                             'physical_network': physical_network})

    def _add_missing_allocatable_vlans(self, session, vlan_ids,
                                       physical_network):
        for vlan_id in sorted(vlan_ids):
            alloc = hyperv_model.VlanAllocation(
                physical_network, vlan_id)
            session.add(alloc)

    def _remove_non_allocatable_vlans(self, session,
                                      physical_network,
                                      vlan_ids,
                                      allocations):
        if physical_network in allocations:
            for alloc in allocations[physical_network]:
                try:
                    # see if vlan is allocatable
                    vlan_ids.remove(alloc.vlan_id)
                except KeyError:
                    # it's not allocatable, so check if its allocated
                    if not alloc.allocated:
                        # it's not, so remove it from table
                        LOG.debug(_(
                            "Removing vlan %(vlan_id)s on "
                            "physical network "
                            "%(physical_network)s from pool"),
                            {'vlan_id': alloc.vlan_id,
                                'physical_network': physical_network})
                        session.delete(alloc)
            del allocations[physical_network]

    def _remove_unconfigured_vlans(self, session, allocations):
        for allocs in allocations.itervalues():
            for alloc in allocs:
                if not alloc.allocated:
                    LOG.debug(_("Removing vlan %(vlan_id)s on physical "
                                "network %(physical_network)s from pool"),
                              {'vlan_id': alloc.vlan_id,
                               'physical_network': alloc.physical_network})
                    session.delete(alloc)

    def sync_vlan_allocations(self, network_vlan_ranges):
        """Synchronize vlan_allocations table with configured VLAN ranges."""

        session = db_api.get_session()
        with session.begin():
            # get existing allocations for all physical networks
            allocations = dict()
            allocs_q = session.query(hyperv_model.VlanAllocation)
            for alloc in allocs_q:
                allocations.setdefault(alloc.physical_network,
                                       set()).add(alloc)

            # process vlan ranges for each configured physical network
            for physical_network, vlan_ranges in network_vlan_ranges.items():
                # determine current configured allocatable vlans for this
                # physical network
                vlan_ids = set()
                for vlan_range in vlan_ranges:
                    vlan_ids |= set(xrange(vlan_range[0], vlan_range[1] + 1))

                # remove from table unallocated vlans not currently allocatable
                self._remove_non_allocatable_vlans(session,
                                                   physical_network,
                                                   vlan_ids,
                                                   allocations)

                # add missing allocatable vlans to table
                self._add_missing_allocatable_vlans(session, vlan_ids,
                                                    physical_network)

            # remove from table unallocated vlans for any unconfigured physical
            # networks
            self._remove_unconfigured_vlans(session, allocations)
